@@ -1,28 +1,31 @@
-"""Extra, independent code, merely arbitrarily stuffed
-here. This is not part of mini-buildd.
+"""
+Extra, independent code, merely arbitrarily stuffed here.
+
+This is not part of mini-buildd.
 
 """
 
-# -*- coding: utf-8 -*-
-from __future__ import unicode_literals
-from __future__ import absolute_import
-
-import urllib2
+import urllib.request
+import urllib.parse
+import urllib.error
 
 import bs4
 
 
-class DebianPackageTracker(object):
-    """Get (some) source package information from the Debian Package
-    Tracker. As long as there is no proper API, we do this the
+class DebianPackageTracker():
+    """
+    Get (some) source package information from the Debian Package Tracker.
+
+    As long as there is no proper API, we do this the
     hacky way parsing the HTML via bs4 - so this will most
     definitely break at some point. Use at own discretion.
     """
+
     def __init__(self, src_package, tracker_url="https://tracker.debian.org"):
         self.info = {}
 
         pkg_url = "{b}/pkg/{p}".format(b=tracker_url, p=src_package)
-        soup = bs4.BeautifulSoup(urllib2.urlopen(pkg_url).read())
+        soup = bs4.BeautifulSoup(urllib.request.urlopen(pkg_url).read())
         version_tags = soup.findAll("span", {"class": "versions-repository"})
         for d in version_tags:
             codename = d['title'].split("(")[1].split(" ")[0].translate({ord(")"): None})
