@@ -1,7 +1,3 @@
-# -*- coding: utf-8 -*-
-from __future__ import unicode_literals
-from __future__ import absolute_import
-
 import os
 import datetime
 import shutil
@@ -10,12 +6,11 @@ import re
 import subprocess
 import logging
 
-import pytz
-
 import django.utils.timezone
 
-import mini_buildd.setup
+import mini_buildd.config
 import mini_buildd.misc
+import mini_buildd.net
 import mini_buildd.call
 import mini_buildd.changes
 
@@ -31,7 +26,7 @@ class Build(mini_buildd.misc.Status):
     UPLOADED = 10
 
     def __init__(self, breq, gnupg, sbuild_jobs):
-        super(Build, self).__init__(
+        super().__init__(
             stati={self.FAILED: "FAILED",
                    self.CHECKING: "CHECKING",
                    self.BUILDING: "BUILDING",
@@ -45,7 +40,7 @@ class Build(mini_buildd.misc.Status):
         self._build_dir = self._breq.get_spool_dir()
         self._chroot = "mini-buildd-{d}-{a}".format(d=self._breq["Base-Distribution"], a=self.architecture)
         # Always generate the now out-of-chroot 'libdir' (needed for ccache, ...).
-        mini_buildd.misc.mkdirs(mini_buildd.misc.chroot_libdir_path(self._breq["Base-Distribution"], self.architecture))
+        os.makedirs(mini_buildd.misc.chroot_libdir_path(self._breq["Base-Distribution"], self.architecture), exist_ok=True)
 
         self._bres = breq.gen_buildresult()
 
@@ -61,7 +56,7 @@ class Build(mini_buildd.misc.Status):
 
         self.live_buildlog_url = breq.get_live_buildlog_loc()
 
-    def __unicode__(self):
+    def __str__(self):
         date_format = "%Y-%b-%d %H:%M:%S"
         return "{s}: [{h}] {k} ({c}): Started {start} ({took} seconds), uploaded {uploaded}: {desc}".format(
             s=self.status,
@@ -101,25 +96,21 @@ class Build(mini_buildd.misc.Status):
     def sbuildrc_path(self):
         return os.path.join(self._build_dir, ".sbuildrc")
 
-    def _get_started_stamp(self):
+    def _get_started_stamp(self):  # pylint: disable=inconsistent-return-statements
         if os.path.exists(self.sbuildrc_path):
-            return datetime.datetime.fromtimestamp(os.path.getmtime(self.sbuildrc_path), tz=pytz.utc)
+            return datetime.datetime.fromtimestamp(os.path.getmtime(self.sbuildrc_path), tz=datetime.timezone.utc)
 
-    def _get_built_stamp(self):
+    def _get_built_stamp(self):  # pylint: disable=inconsistent-return-statements
         if os.path.exists(self._bres.file_path):
-            return datetime.datetime.fromtimestamp(os.path.getmtime(self._bres.file_path), tz=pytz.utc)
+            return datetime.datetime.fromtimestamp(os.path.getmtime(self._bres.file_path), tz=datetime.timezone.utc)
 
     @property
     def took(self):
-        return round(mini_buildd.misc.timedelta_total_seconds(self.built - self.started), 1) if self.built else "n/a"
+        return round((self.built - self.started).total_seconds(), 1) if self.built else "n/a"
 
     def _generate_sbuildrc(self):
-        """
-        Generate .sbuildrc for a build request (not all is configurable via switches, unfortunately).
-        """
-        mini_buildd.misc.ConfFile(
-            self.sbuildrc_path,
-            """\
+        """Generate .sbuildrc for a build request (not all is configurable via switches, unfortunately)."""
+        tpl = """\
 # We update sources.list on the fly via chroot-setup commands;
 # this update occurs before, so we don't need it.
 $apt_update = 0;
@@ -131,8 +122,12 @@ $apt_allow_unauthenticated = {apt_allow_unauthenticated};
 
 # don't remove this, Perl needs it:
 1;
-""".format(apt_allow_unauthenticated=self._breq["Apt-Allow-Unauthenticated"],
-           custom_snippet=mini_buildd.misc.open_utf8(os.path.join(self._build_dir, "sbuildrc_snippet"), 'rb').read())).save()
+"""
+        with mini_buildd.misc.open_utf8(os.path.join(self._build_dir, "sbuildrc_snippet")) as cs:
+            mini_buildd.misc.ConfFile(
+                self.sbuildrc_path,
+                tpl.format(apt_allow_unauthenticated=self._breq["Apt-Allow-Unauthenticated"],
+                           custom_snippet=cs.read())).save()
 
     def _buildlog_to_buildresult(self, buildlog):
         """
@@ -147,27 +142,14 @@ $apt_allow_unauthenticated = {apt_allow_unauthenticated};
         better way to get these values.
         """
         regex = re.compile("^(Status|Lintian): [^ ]+$")
-        with open(buildlog) as f:
-            for l in f:
-                if regex.match(l):
-                    LOG.debug("Build log line detected as build status: {l}".format(l=l.strip()))
-                    s = l.split(":")
+        with open(buildlog, encoding=mini_buildd.config.CHAR_ENCODING, errors="replace") as f:
+            for line in f:
+                if regex.match(line):
+                    LOG.debug("Build log line detected as build status: {line}".format(line=line.strip()))
+                    s = line.split(":")
                     self._bres["Sbuild-" + s[0]] = s[1].strip()
 
     def build(self):
-        """
-        .. note:: OBSOLETED SUDO WORKAROUND for http://bugs.debian.org/cgi-bin/bugreport.cgi?bug=608840
-
-            The workaround has been removed, except for the "/etc/sudoers" cleanup (search below).
-
-            This is still there so that users who do not re-create their chroots for whatever reason at
-            least keep the same level of security as before.
-
-            Since some version of sudo, upgrading the package with missing /etc/sudoers fails, so we now
-            additionally touch this, so /etc/sudoers exist as empty file.
-
-            This left over will be removed for the 1.2.x version.
-        """
         self._breq.untar(path=self._build_dir)
         self._generate_sbuildrc()
         self.started = self._get_started_stamp()
@@ -175,12 +157,10 @@ $apt_allow_unauthenticated = {apt_allow_unauthenticated};
         # Check if we need to support a https apt transport
         sources_list_file = "{p}/apt_sources.list".format(p=self._build_dir)
         apt_transports = []
-        with mini_buildd.misc.open_utf8(sources_list_file) as sources_list:
-            for apt_line in sources_list:
-                if re.match("deb https", apt_line):
-                    apt_transports = ["--chroot-setup-command", "apt-get --yes --option=APT::Install-Recommends=false install ca-certificates apt-transport-https"]
-                    LOG.info("{p}: Found https source: {l}".format(p=self.key, l=apt_line))
-                    break
+
+        sources_list_has_https = mini_buildd.misc.sources_list_has_https(sources_list_file)
+        if sources_list_has_https:
+            apt_transports = ["--chroot-setup-command", "apt-get --yes --option=APT::Install-Recommends=false install ca-certificates apt-transport-https"]
 
         sbuild_cmd = ["sbuild",
                       "--dist", self.distribution,
@@ -199,11 +179,11 @@ $apt_allow_unauthenticated = {apt_allow_unauthenticated};
                        "--chroot-setup-command", "cp {p}/apt_preferences /etc/apt/preferences".format(p=self._build_dir),
                        "--chroot-setup-command", "cat /etc/apt/preferences",
                        "--chroot-setup-command", "apt-key add {p}/apt_keys".format(p=self._build_dir),
+                       "--chroot-setup-command", "cp -v {p}/ssl_cert /usr/local/share/ca-certificates/mini-buildd-repo.crt".format(p=self._build_dir) if sources_list_has_https else "echo 'No https: Skipping certificate installation.'",
+                       "--chroot-setup-command", "/usr/sbin/update-ca-certificates" if sources_list_has_https else "echo 'No https: Not updating certificates.'",
                        "--chroot-setup-command", "apt-get --option=Acquire::Languages=none update",
                        "--chroot-setup-command", "{p}/chroot_setup_script".format(p=self._build_dir),
                        "--chroot-setup-command", "cat {p}/chroot_setup_script".format(p=self._build_dir),
-                       "--chroot-setup-command", "rm -v -f /etc/sudoers",
-                       "--chroot-setup-command", "touch /etc/sudoers",
                        "--chroot-setup-command", "apt-cache policy",
                        "--build-dep-resolver", self._breq["Build-Dep-Resolver"],
                        "--keyid", self._gnupg.get_first_sec_key(),
@@ -216,25 +196,26 @@ $apt_allow_unauthenticated = {apt_allow_unauthenticated};
         else:
             sbuild_cmd += ["--no-arch-all"]  # Be sure to set explicitly: sbuild >= 0.77 does not seem to use this as default any more?
 
+        sbuild_cmd += ["--jobs={j}".format(j=self._sbuild_jobs)]
+
         if "Run-Lintian" in self._breq:
             sbuild_cmd += ["--run-lintian"]
-            # Be sure not to use --suppress-tags when its not availaible (only >=squeeze).
+            # Be sure not to use --suppress-tags when its not available (only >=squeeze).
             if mini_buildd.misc.Distribution(self.distribution).has_lintian_suppress():
                 sbuild_cmd += ["--lintian-opts", "--suppress-tags=bad-distribution-in-changes-file"]
             sbuild_cmd += ["--lintian-opts", self._breq["Run-Lintian"]]
         else:
             sbuild_cmd += ["--no-run-lintian"]
 
-        if "sbuild" in mini_buildd.setup.DEBUG:
+        if "sbuild" in mini_buildd.config.DEBUG:
             sbuild_cmd += ["--debug"]
 
         sbuild_cmd += [self._breq.dsc_name]
 
         # Actually run sbuild
-        mini_buildd.call.sbuild_keys_workaround()
         buildlog = os.path.join(self._build_dir, self._breq.buildlog_name)
-        live_buildlog = os.path.join(mini_buildd.setup.SPOOL_DIR, self._breq.live_buildlog_name)
-        with open(buildlog, "w+") as l:
+        live_buildlog = os.path.join(mini_buildd.config.SPOOL_DIR, self._breq.live_buildlog_name)
+        with open(buildlog, "w+") as buildlog_file:
             LOG.info("Adding live buildlog: {b}".format(b=live_buildlog))
             # The spool id/hash might the very same (retry a failed build, for example) as a previous one. So we need to be sure to remove before linking.
             # Note that this currently should never really happen as python's tarball abstraction cannot produce reproducible tarballs (https://bugs.python.org/issue24465).
@@ -244,13 +225,13 @@ $apt_allow_unauthenticated = {apt_allow_unauthenticated};
             sbuild_call = mini_buildd.call.Call(sbuild_cmd,
                                                 cwd=self._build_dir,
                                                 env=mini_buildd.call.taint_env({"HOME": self._build_dir,
-                                                                                "GNUPGHOME": os.path.join(mini_buildd.setup.HOME_DIR, ".gnupg"),
-                                                                                "DEB_BUILD_OPTIONS": "{d} parallel={j}".format(d=self._breq.get("Deb-Build-Options", ""), j=self._sbuild_jobs)}),
-                                                stdout=l, stderr=subprocess.STDOUT)
-            retval = sbuild_call.retval
+                                                                                "GNUPGHOME": os.path.join(mini_buildd.config.HOME_DIR, ".gnupg"),
+                                                                                "DEB_BUILD_OPTIONS": self._breq.get("Deb-Build-Options", "")}),
+                                                stdout=buildlog_file, stderr=subprocess.STDOUT)
+            retval = sbuild_call.result.returncode
 
         # Add build results to build request object
-        self._bres["Sbuildretval"] = unicode(retval)
+        self._bres["Sbuildretval"] = str(retval)
         self._buildlog_to_buildresult(buildlog)
 
         LOG.info("{p}: Sbuild finished: Sbuildretval={r}, Status={s}".format(p=self.key, r=retval, s=self._bres.get("Sbuild-Status")))
@@ -268,8 +249,8 @@ $apt_allow_unauthenticated = {apt_allow_unauthenticated};
         self.built = self._get_built_stamp()
 
     def upload(self):
-        hopo = mini_buildd.misc.HoPo(self.upload_result_to)
-        self._bres.upload(hopo)
+        ep = mini_buildd.net.ClientEndpoint(mini_buildd.net.Endpoint.hopo2desc(self.upload_result_to, server=False), mini_buildd.net.Protocol.FTP)
+        self._bres.upload(ep)
         self.uploaded = django.utils.timezone.now()
 
     def clean(self):
@@ -278,14 +259,13 @@ $apt_allow_unauthenticated = {apt_allow_unauthenticated};
 
 
 class LastBuild(mini_buildd.misc.API):
-    """
-    Subset of 'Build' for pickled statistics.
-    """
+    """Subset of 'Build' for pickled statistics."""
+
     __API__ = -99
 
     def __init__(self, build):
-        super(LastBuild, self).__init__()
-        self.identity = build.__unicode__()
+        super().__init__()
+        self.identity = build.__str__()
         self.status = build.status
         self.status_desc = build.status_desc
 
@@ -299,38 +279,34 @@ class LastBuild(mini_buildd.misc.API):
         self.upload_result_to = build.upload_result_to
         self.live_buildlog_url = build.live_buildlog_url
 
-    def __unicode__(self):
+    def __str__(self):
         return self.identity
 
 
 def _expire_live_buildlogs(**kwargs):
-    """
-    Expire live buildlogs older than timedelta. Arguments are given as-is to the datetime.timedelta constructor.
-    """
+    """Expire live buildlogs older than timedelta. Arguments are given as-is to the datetime.timedelta constructor."""
     valid_until = django.utils.timezone.now() - datetime.timedelta(**kwargs)
-    for buildlog in glob.glob(os.path.join(mini_buildd.setup.SPOOL_DIR, "*.buildlog")):
+    for buildlog in glob.glob(os.path.join(mini_buildd.config.SPOOL_DIR, "*.buildlog")):
         LOG.debug("Checking if live build log is older than {d}: {logfile}".format(d=valid_until, logfile=buildlog))
-        if datetime.datetime.fromtimestamp(os.path.getmtime(buildlog), tz=pytz.utc) < valid_until:
+        if datetime.datetime.fromtimestamp(os.path.getmtime(buildlog), tz=datetime.timezone.utc) < valid_until:
             LOG.info("Expiring live build log: {logfile}".format(logfile=buildlog))
             os.unlink(buildlog)
 
 
 def build_close(daemon, build):
-    """
-    Close build. Just continue on errors, but log them; guarantee to remove it from the builds dict.
-    """
+    """Close build. Just continue on errors, but log them; guarantee to remove it from the builds dict."""
     try:
         build.clean()
         daemon.last_builds.appendleft(LastBuild(build))
         _expire_live_buildlogs(days=5)
-    except Exception as e:
-        mini_buildd.setup.log_exception(LOG, "Error closing build '{p}'".format(p=build.key), e, level=logging.CRITICAL)
+    except BaseException as e:
+        mini_buildd.config.log_exception(LOG, "Error closing build '{p}'".format(p=build.key), e, level=logging.CRITICAL)
     finally:
         if build.key in daemon.builds:
             del daemon.builds[build.key]
 
 
-def build(daemon_, breq):
+def run_build(daemon_, breq):
     build = None
     try:
         # First, get build object. This will automagically set the status right.
@@ -350,16 +326,16 @@ def build(daemon_, breq):
         try:
             build.upload()
             build.set_status(build.UPLOADED)
-        except Exception as e:
-            mini_buildd.setup.log_exception(LOG, "Upload failed (retry later)", e, logging.WARN)
-            build.set_status(build.UPLOADING, unicode(e))
+        except BaseException as e:
+            mini_buildd.config.log_exception(LOG, "Upload failed (retry later)", e, logging.WARN)
+            build.set_status(build.UPLOADING, str(e))
 
-    except Exception as e:
+    except BaseException as e:
         # Try to upload failure build result to remote
         if build:
             build.set_status(build.FAILED)
-        breq.upload_failed_buildresult(daemon_.model.mbd_gnupg, mini_buildd.misc.HoPo(breq["Upload-Result-To"]), 101, "builder-failed", e)
-        mini_buildd.setup.log_exception(LOG, "Internal error building", e)
+        breq.upload_failed_buildresult(daemon_.model.mbd_gnupg, mini_buildd.net.ClientEndpoint(mini_buildd.net.Endpoint.hopo2desc(breq["Upload-Result-To"]), server=False), 101, "builder-failed", e)
+        mini_buildd.config.log_exception(LOG, "Internal error building", e)
 
     finally:
         if build:
@@ -376,7 +352,7 @@ def run(daemon_):
         LOG.info("Builder status: {s}.".format(s=daemon_.build_queue))
 
         mini_buildd.misc.run_as_thread(
-            build,
+            run_build,
             name="building {pkg}".format(pkg=os.path.basename(event)),
             daemon=True,
             daemon_=daemon_,
