@@ -61,7 +61,7 @@ class Call():
             elif not isinstance(self.kwargs[stream], int):
                 self._given_stream[stream] = self.kwargs[stream]
 
-        self.result = subprocess.run(self.call, **self.kwargs)
+        self.result = subprocess.run(self.call, **self.kwargs, check=False)
 
         # Convenience 'label' for log output
         self.label = "{p} {c}..".format(p="#" if run_as_root else "?", c=call[0])
