@@ -1,9 +1,3 @@
-# -*- coding: utf-8 -*-
-# django false-positives:
-# pylint: disable=unexpected-keyword-arg,no-value-for-parameter
-from __future__ import unicode_literals
-from __future__ import absolute_import
-
 import os
 import stat
 import glob
@@ -12,13 +6,15 @@ import logging
 import tarfile
 import socket
 import ftplib
+import urllib.parse
 import re
 import contextlib
 
 import debian.deb822
 
-import mini_buildd.setup
+import mini_buildd.config
 import mini_buildd.misc
+import mini_buildd.net
 import mini_buildd.gnupg
 
 import mini_buildd.models.repository
@@ -27,18 +23,19 @@ import mini_buildd.models.gnupg
 LOG = logging.getLogger(__name__)
 
 
-class Changes(debian.deb822.Changes):
-    class Options(object):
+class Changes(debian.deb822.Changes):  # pylint: disable=too-many-ancestors
+    class Options():
         """
         Uploader options in changes.
 
-        >>> "{}".format(Changes("./examples/doctests/changes.options").options)
-        u"auto-ports=[u'jessie-test-unstable', u'squeeze-test-snasphot'], ignore-lintian=True, ignore-lintian[i386]=False, internal-apt-priority=543, run-lintian=True, run-lintian[i386]=False"
+        >>> "{}".format(Changes("test-data/changes.options").options)
+        "auto-ports=['jessie-test-unstable', 'squeeze-test-snasphot'], ignore-lintian=True, ignore-lintian[i386]=False, internal-apt-priority=543, run-lintian=True, run-lintian[i386]=False"
 
-        >>> "{}".format(Changes("./examples/doctests/changes.magic").options)
-        u"auto-ports=[u'jessie-test-unstable', u'squeeze-test-snasphot'], ignore-lintian=True"
+        >>> "{}".format(Changes("test-data/changes.magic").options)
+        "auto-ports=['jessie-test-unstable', 'squeeze-test-snasphot'], ignore-lintian=True"
         """
-        class Bool(object):
+
+        class Bool():
             _TRUE = ["true", "1"]
             _FALSE = ["false", "0"]
             _VALID = _TRUE + _FALSE
@@ -48,11 +45,11 @@ class Changes(debian.deb822.Changes):
                     raise Exception("Bool value must be one of {}".format(",".join(self._VALID)))
                 self.value = raw_value.lower() in self._TRUE
 
-        class Int(object):
+        class Int():
             def __init__(self, raw_value):
                 self.value = int(raw_value)
 
-        class CSV(object):
+        class CSV():
             def __init__(self, raw_value):
                 self.value = raw_value.split(",")
 
@@ -64,7 +61,8 @@ class Changes(debian.deb822.Changes):
 
         @classmethod
         def _get_top_changes(cls, upload_changes):
-            """Filter only the first block from the changes (changelog) entry.
+            """
+            Filter only the first block from the changes (changelog) entry.
 
             Upload changes may include multiple version blocks from
             the changelog (internal porting does it, for example),
@@ -81,11 +79,9 @@ class Changes(debian.deb822.Changes):
             return result
 
         def _compat_parse_magic(self):
-            """
-            Compat parse support for old style "magic" options.
-            """
+            """Compat parse support for old style "magic" options."""
             def warning(magic, option):
-                LOG.warn("Deprecated \"magic\" option \"{m}\" found. Please use new-style option \"{o}\" instead (see user manual).".format(m=magic, o=option))
+                LOG.warning("Deprecated \"magic\" option \"{m}\" found. Please use new-style option \"{o}\" instead (see user manual).".format(m=magic, o=option))
 
             magic_auto_backports = re.search(r"\*\s*MINI_BUILDD:\s*AUTO_BACKPORTS:\s*([^*.\[\]]+)", self._top_changes)
             if magic_auto_backports:
@@ -106,7 +102,7 @@ class Changes(debian.deb822.Changes):
 
             self._compat_parse_magic()
 
-        def __unicode__(self):
+        def __str__(self):
             return ", ".join("{k}={v}".format(k=key, v=value) for key, value in sorted(self._options.items()))
 
         def _set(self, key, raw_value):
@@ -114,11 +110,11 @@ class Changes(debian.deb822.Changes):
             value = re.sub(r"\s+", "", raw_value)
 
             # Validity check for key
-            if base_key not in self._OPTIONS.keys():
+            if base_key not in list(self._OPTIONS.keys()):
                 raise Exception("Unknown upload option: {k}.".format(k=key))
 
             # Duplicity check
-            if key in self._options.keys():
+            if key in list(self._options.keys()):
                 raise Exception("Duplicate upload option: {k}.".format(k=key))
 
             # Value conversion check
@@ -133,11 +129,9 @@ class Changes(debian.deb822.Changes):
             LOG.debug("Upload option set: {k}=\"{v}\"".format(k=key, v=value))
 
         def get(self, key, alt=None, default=None):
-            """
-            Get first existing option value in this order: key[a], key, default.
-            """
+            """Get first existing option value in this order: key[a], key, default."""
             # Validity check for key
-            if key not in self._OPTIONS.keys():
+            if key not in list(self._OPTIONS.keys()):
                 raise Exception("Internal error: Upload Options: Unknown key used for get(): {k}.".format(k=key))
 
             if alt:
@@ -171,7 +165,11 @@ class Changes(debian.deb822.Changes):
         # Instance might be produced from a temporary file, so we need to save the hash now.
         self._spool_hash = self._spool_hash_from_file()
 
-        super(Changes, self).__init__([] if self._new else open(file_path))
+        if self._new:
+            super().__init__([])
+        else:
+            with mini_buildd.misc.open_utf8(file_path) as cf:
+                super().__init__(cf)
 
         self._options = None
         if self.BUILDREQUEST_RE.match(self._file_name):
@@ -182,25 +180,22 @@ class Changes(debian.deb822.Changes):
             self._type = self.TYPE_DEFAULT
 
         # Be sure base dir is always available
-        mini_buildd.misc.mkdirs(os.path.dirname(file_path))
+        os.makedirs(os.path.dirname(file_path), exist_ok=True)
 
         # This is just for stat/display purposes
         self.remote_http_url = None
         self.live_buildlog_url = None
 
-    def __unicode__(self):
+    def __str__(self):
         if self.type == self.TYPE_BREQ:
             return "Buildrequest from '{h}': {i}".format(h=self.get("Upload-Result-To"), i=self.get_pkg_id(with_arch=True))
-        elif self.type == self.TYPE_BRES:
+        if self.type == self.TYPE_BRES:
             return "Buildresult from '{h}': {i}".format(h=self.get("Built-By"), i=self.get_pkg_id(with_arch=True))
-        else:
-            return "User upload: {i}".format(i=self.get_pkg_id())
+        return "User upload: {i}".format(i=self.get_pkg_id())
 
     @property
     def options(self):
-        """
-        .. note:: We can't parse this in constructor currently: Upload option error handling won't work properly, exceptions triggered too early in packager.py.
-        """
+        """note:: We can't parse this in constructor currently: Upload option error handling won't work properly, exceptions triggered too early in packager.py."""
         if not self._options:
             self._options = self.Options(self)
         return self._options
@@ -218,13 +213,13 @@ class Changes(debian.deb822.Changes):
         mini-buildd types.
 
         >>> Changes.gen_changes_file_name("mypkg", "1.2.3-1", "mips")
-        u'mypkg_1.2.3-1_mips.changes'
+        'mypkg_1.2.3-1_mips.changes'
         >>> Changes.gen_changes_file_name("mypkg", "7:1.2.3-1", "mips")
-        u'mypkg_1.2.3-1_mips.changes'
+        'mypkg_1.2.3-1_mips.changes'
         >>> Changes.gen_changes_file_name("mypkg", "7:1.2.3-1", "mips", mbd_type=Changes.TYPE_BREQ)
-        u'mypkg_1.2.3-1_mini-buildd-buildrequest_mips.changes'
+        'mypkg_1.2.3-1_mini-buildd-buildrequest_mips.changes'
         >>> Changes.gen_changes_file_name("mypkg", "7:1.2.3-1", "mips", mbd_type=Changes.TYPE_BRES)
-        u'mypkg_1.2.3-1_mini-buildd-buildresult_mips.changes'
+        'mypkg_1.2.3-1_mini-buildd-buildresult_mips.changes'
         """
         return "{p}_{v}{x}_{a}.changes".format(p=package,
                                                v=mini_buildd.misc.strip_epoch(version),
@@ -248,7 +243,7 @@ class Changes(debian.deb822.Changes):
 
     @property
     def bres_stat(self):
-        return "Build={b}, Lintian={l}".format(b=self.get("Sbuild-Status"), l=self.get("Sbuild-Lintian"))
+        return "Build={build}, Lintian={lintian}".format(build=self.get("Sbuild-Status"), lintian=self.get("Sbuild-Lintian"))
 
     @property
     def file_name(self):
@@ -270,10 +265,12 @@ class Changes(debian.deb822.Changes):
         return "/mini_buildd/live-buildlogs/{logfile}".format(logfile=self.live_buildlog_name)
 
     def get_live_buildlog_url(self, base_url):
-        return "{base_url}/{loc}".format(base_url=base_url, loc=self.get_live_buildlog_loc())
+        return urllib.parse.urljoin(base_url, self.get_live_buildlog_loc())
 
     def get_pkglog_dir(self, installed, relative=True):
         """
+        Get package log dir.
+
         Package log path for this changes file: REPOID/[_failed]/PACKAGE/VERSION/ARCH
 
         In case the changes is bogus (i.e, cannot produce a
@@ -287,8 +284,8 @@ class Changes(debian.deb822.Changes):
                                                     self["Version"],
                                                     architecture=self["Architecture"],
                                                     relative=relative)
-        except Exception as e:
-            mini_buildd.setup.log_exception(LOG, "No package log dir for bogus changes: {f}".format(f=self.file_name), e, logging.DEBUG)
+        except BaseException as e:
+            mini_buildd.config.log_exception(LOG, "No package log dir for bogus changes: {f}".format(f=self.file_name), e, logging.DEBUG)
 
     def is_new(self):
         return self._new
@@ -297,7 +294,7 @@ class Changes(debian.deb822.Changes):
         return "{type}-{hash}".format(type=self.TYPE2NAME[self._type], hash=self._spool_hash)
 
     def get_spool_dir(self):
-        return os.path.join(mini_buildd.setup.SPOOL_DIR, self.get_spool_id())
+        return os.path.join(mini_buildd.config.SPOOL_DIR, self.get_spool_id())
 
     def get_pkg_id(self, with_arch=False, arch_separator=":"):
         pkg_id = "{s}_{v}".format(s=self["Source"], v=self["Version"])
@@ -330,34 +327,40 @@ class Changes(debian.deb822.Changes):
         """
         try:
             LOG.info("Saving changes: {f}".format(f=self._file_path))
-            self.dump(fd=open(self._file_path, "w+"), encoding=mini_buildd.setup.CHAR_ENCODING)
+            with open(self._file_path, "w+", encoding=mini_buildd.config.CHAR_ENCODING) as f:
+                f.write(self.dump())
+
             LOG.info("Signing changes: {f}".format(f=self._file_path))
             if gnupg:
                 gnupg.sign(self._file_path)
             self._spool_hash = self._spool_hash_from_file()
-        except:
+        except BaseException:
             # Existence of the file name is used as flag
             if os.path.exists(self._file_path):
                 os.remove(self._file_path)
             raise
 
-    def upload(self, hopo):
+    def upload(self, endpoint):
         upload = os.path.splitext(self._file_path)[0] + ".upload"
         if os.path.exists(upload):
-            LOG.info("FTP: '{f}' already uploaded to '{h}'...".format(f=self._file_name, h=mini_buildd.misc.open_utf8(upload).read()))
+            with mini_buildd.misc.open_utf8(upload) as uf:
+                LOG.info("FTP: '{f}' already uploaded to '{h}'...".format(f=self._file_name, h=uf.read()))
         else:
             ftp = ftplib.FTP()
-            ftp.connect(hopo.host, hopo.port)
+            host, port = endpoint.option("host"), endpoint.option("port")
+            ftp.connect(host, int(port))
             ftp.login()
             ftp.cwd("/incoming")
             for fd in self.get_files() + [{"name": self._file_name}]:
                 f = fd["name"]
                 LOG.debug("FTP: Uploading file: '{f}'".format(f=f))
-                ftp.storbinary("STOR {f}".format(f=f), open(os.path.join(os.path.dirname(self._file_path), f), "rb"))
-            mini_buildd.misc.open_utf8(upload, "w").write("{h}:{p}".format(h=hopo.host, p=hopo.port))
-            LOG.info("FTP: '{f}' uploaded to '{h}'...".format(f=self._file_name, h=hopo.host))
+                with open(os.path.join(os.path.dirname(self._file_path), f), "rb") as fi:
+                    ftp.storbinary("STOR {f}".format(f=f), fi)
+            with mini_buildd.misc.open_utf8(upload, "w") as fi:
+                fi.write("{h}:{p}".format(h=host, p=port))
+            LOG.info("FTP: '{f}' uploaded to '{h}'...".format(f=self._file_name, h=host))
 
-    def upload_buildrequest(self, local_hopo):
+    def upload_buildrequest(self, local_endpoint):
         arch = self["Architecture"]
         codename = self["Base-Distribution"]
 
@@ -365,19 +368,20 @@ class Changes(debian.deb822.Changes):
 
         def add_remote(remote, update):
             status = remote.mbd_get_status(update)
+            status.url = remote.mbd_http2url()  # Not cool: Monkey patching status for url
             if status.running and status.has_chroot(codename, arch):
                 remotes[status.load] = status
-                LOG.debug("Remote[{l}]={r}".format(l=status.load, r=remote))
+                LOG.debug("Remote[{load}]={remote}".format(load=status.load, remote=remote))
 
         def check_remote(remote):
             try:
                 mini_buildd.models.gnupg.Remote.Admin.mbd_check(None, remote, force=True)
                 add_remote(remote, False)
-            except Exception as e:
-                mini_buildd.setup.log_exception(LOG, "Builder check failed", e, logging.WARNING)
+            except BaseException as e:
+                mini_buildd.config.log_exception(LOG, "Builder check failed", e, logging.WARNING)
 
         # Always add our own instance as pseudo remote first
-        add_remote(mini_buildd.models.gnupg.Remote(http=local_hopo.string), True)
+        add_remote(mini_buildd.models.gnupg.Remote(http="{proto}:{hopo}".format(proto=mini_buildd.config.HTTPD_ENDPOINTS[0].url_scheme, hopo=local_endpoint.hopo())), True)
 
         # Check all active or auto-deactivated remotes
         for r in mini_buildd.models.gnupg.Remote.mbd_get_active_or_auto_reactivate():
@@ -388,13 +392,12 @@ class Changes(debian.deb822.Changes):
 
         for _load, remote in sorted(remotes.items()):
             try:
-                self.upload(mini_buildd.misc.HoPo(remote.ftp))
-                remote_url = "http://" + remote.http
-                self.remote_http_url = remote_url
-                self.live_buildlog_url = self.get_live_buildlog_url(base_url=remote_url)
+                self.upload(mini_buildd.net.ClientEndpoint(mini_buildd.net.Endpoint.hopo2desc(remote.ftp, server=False), mini_buildd.net.Protocol.FTP))
+                self.remote_http_url = remote.url
+                self.live_buildlog_url = self.get_live_buildlog_url(base_url=remote.url)
                 return
-            except Exception as e:
-                mini_buildd.setup.log_exception(LOG, "Uploading to '{h}' failed".format(h=remote.ftp), e, logging.WARNING)
+            except BaseException as e:
+                mini_buildd.config.log_exception(LOG, "Uploading to '{h}' failed".format(h=remote.ftp), e, logging.WARNING)
 
         raise Exception("Buildrequest upload failed for {a}/{c}".format(a=arch, c=codename))
 
@@ -435,7 +438,7 @@ class Changes(debian.deb822.Changes):
         if logdir and not os.path.exists(logdir):
             os.makedirs(logdir)
 
-        LOG.info("Moving changes to package log: '{f}'->'{l}'".format(f=self._file_path, l=logdir))
+        LOG.info("Moving changes to package log: '{f}'->'{d}'".format(f=self._file_path, d=logdir))
         for fd in [{"name": self._file_name}] + self.get_files():
             f = fd["name"]
             f_abs = os.path.join(os.path.dirname(self._file_path), f)
@@ -458,14 +461,18 @@ class Changes(debian.deb822.Changes):
     def gen_buildrequests(self, daemon, repository, dist, suite_option):
         """
         Build buildrequest files for all architectures.
-        """
 
+        .. todo:: **IDEA**: gen_buildrequests(): Instead of tar'ing ourselves (uploaded changes)
+                  with exceptions (.deb, .buildinfo, .changes), add the *.dsc* and its files only!
+        """
         # Extra check on all DSC/source package files
         # - Check md5 against possible pool files.
         # - Add missing from pool (i.e., orig.tar.gz).
         # - make sure all files from dsc are actually available
         files_from_pool = []
-        dsc = debian.deb822.Dsc(open(self.dsc_file_name))
+        with open(self.dsc_file_name) as dsc_file:
+            dsc = debian.deb822.Dsc(dsc_file)
+
         for f in dsc["Files"]:
             in_changes = f["name"] in self.get_files(key="name")
             from_pool = False
@@ -495,28 +502,34 @@ class Changes(debian.deb822.Changes):
                 for v in ["Source", "Version"]:
                     breq[v] = self[v]
 
-                # Generate sources.list et.al. to be used
-                mini_buildd.misc.open_utf8(os.path.join(path, "apt_sources.list"), "w").write(dist.mbd_get_apt_sources_list(repository, suite_option))
-                mini_buildd.misc.open_utf8(os.path.join(path, "apt_preferences"), "w").write(dist.mbd_get_apt_preferences(repository, suite_option, self.options.get("internal-apt-priority")))
-                mini_buildd.misc.open_utf8(os.path.join(path, "apt_keys"), "w").write(repository.mbd_get_apt_keys(dist))
+                # Generate files
                 chroot_setup_script = os.path.join(path, "chroot_setup_script")
-                # Note: For some reason (python, django sqlite, browser?) the text field may be in DOS mode.
-                mini_buildd.misc.open_utf8(chroot_setup_script, "w").write(mini_buildd.misc.fromdos(dist.chroot_setup_script))
-
-                os.chmod(chroot_setup_script, stat.S_IRWXU)
-                mini_buildd.misc.open_utf8(os.path.join(path, "sbuildrc_snippet"), "w").write(dist.mbd_get_sbuildrc_snippet(ao.architecture.name))
+                with mini_buildd.misc.open_utf8(os.path.join(path, "apt_sources.list"), "w") as asl, \
+                     mini_buildd.misc.open_utf8(os.path.join(path, "apt_preferences"), "w") as ap, \
+                     mini_buildd.misc.open_utf8(os.path.join(path, "apt_keys"), "w") as ak, \
+                     mini_buildd.misc.open_utf8(os.path.join(path, "ssl_cert"), "w") as ssl_cert, \
+                     mini_buildd.misc.open_utf8(chroot_setup_script, "w") as css, \
+                     mini_buildd.misc.open_utf8(os.path.join(path, "sbuildrc_snippet"), "w") as src:
+                    asl.write(dist.mbd_get_apt_sources_list(repository, suite_option))
+                    ap.write(dist.mbd_get_apt_preferences(repository, suite_option, self.options.get("internal-apt-priority")))
+                    ak.write(repository.mbd_get_apt_keys(dist))
+                    ssl_cert.write(mini_buildd.config.HTTPD_ENDPOINTS[0].get_certificate())
+                    css.write(mini_buildd.misc.fromdos(dist.chroot_setup_script))  # Note: For some reason (python, django sqlite, browser?) the text field may be in DOS mode.
+                    os.chmod(chroot_setup_script, stat.S_IRWXU)
+                    src.write(dist.mbd_get_sbuildrc_snippet(ao.architecture.name))
 
                 # Generate tar from original changes
                 self.tar(tar_path=breq.file_path + ".tar",
                          add_files=[os.path.join(path, "apt_sources.list"),
                                     os.path.join(path, "apt_preferences"),
                                     os.path.join(path, "apt_keys"),
+                                    os.path.join(path, "ssl_cert"),
                                     chroot_setup_script,
                                     os.path.join(path, "sbuildrc_snippet")] + files_from_pool,
                          exclude_globs=["*.deb", "*.changes", "*.buildinfo"])
                 breq.add_file(breq.file_path + ".tar")
 
-                breq["Upload-Result-To"] = daemon.mbd_get_ftp_hopo().string
+                breq["Upload-Result-To"] = daemon.mbd_get_ftp_endpoint().hopo()
                 breq["Base-Distribution"] = dist.base_source.codename
                 breq["Architecture"] = ao.architecture.name
                 if ao.build_architecture_all:
@@ -553,18 +566,18 @@ class Changes(debian.deb822.Changes):
 
         return bres
 
-    def upload_failed_buildresult(self, gnupg, hopo, retval, status, exception):
+    def upload_failed_buildresult(self, gnupg, endpoint, retval, status, exception):
         with contextlib.closing(mini_buildd.misc.TmpDir()) as t:
             bres = self.gen_buildresult(path=t.tmpdir)
 
-            bres["Sbuildretval"] = unicode(retval)
+            bres["Sbuildretval"] = str(retval)
             bres["Sbuild-Status"] = status
             buildlog = os.path.join(t.tmpdir, self.buildlog_name)
-            with mini_buildd.misc.open_utf8(buildlog, "w+") as l:
-                l.write("""
+            with mini_buildd.misc.open_utf8(buildlog, "w+") as log_file:
+                log_file.write("""
 Host: {h}
 Build request failed: {r} ({s}): {e}
 """.format(h=socket.getfqdn(), r=retval, s=status, e=exception))
             bres.add_file(buildlog)
             bres.save(gnupg)
-            bres.upload(hopo)
+            bres.upload(endpoint)
