@@ -1,25 +1,24 @@
-# -*- coding: utf-8 -*-
-from __future__ import unicode_literals
-from __future__ import absolute_import
-
 import os
 import re
 import time
 import shutil
 import glob
 import threading
-import Queue
+import queue
 import collections
-import urllib2
+import urllib.request
+import urllib.parse
+import urllib.error
 import logging
 
+import django.conf
+
 import debian.deb822
 import debian.changelog
 import debian.debian_support
 
-import django.conf
-
 import mini_buildd.misc
+import mini_buildd.net
 import mini_buildd.call
 import mini_buildd.changes
 import mini_buildd.gnupg
@@ -38,36 +37,33 @@ LOG = logging.getLogger(__name__)
 
 
 class Changelog(debian.changelog.Changelog):
-    """
+    r"""
     Changelog class with some extra functions.
 
-    >>> cl = Changelog(mini_buildd.misc.open_utf8("./examples/doctests/changelog"), max_blocks=100)
+    >>> cl = Changelog(mini_buildd.misc.open_utf8("test-data/changelog"), max_blocks=100)
     >>> cl.find_first_not("mini-buildd@buildd.intra")
-    (u'Stephan S\\xfcrken <absurd@debian.org>', u'1.0.0-2')
+    ('Stephan Sürken <absurd@debian.org>', '1.0.0-2')
 
-    >>> cl = Changelog(mini_buildd.misc.open_utf8("./examples/doctests/changelog.ported"), max_blocks=100)
+    >>> cl = Changelog(mini_buildd.misc.open_utf8("test-data/changelog.ported"), max_blocks=100)
     >>> cl.find_first_not("mini-buildd@buildd.intra")
-    (u'Stephan S\\xfcrken <absurd@debian.org>', u'1.0.0-2')
+    ('Stephan Sürken <absurd@debian.org>', '1.0.0-2')
 
-    >>> cl = Changelog(mini_buildd.misc.open_utf8("./examples/doctests/changelog.oneblock"), max_blocks=100)
+    >>> cl = Changelog(mini_buildd.misc.open_utf8("test-data/changelog.oneblock"), max_blocks=100)
     >>> cl.find_first_not("mini-buildd@buildd.intra")
-    (u'Stephan S\\xfcrken <absurd@debian.org>', u'1.0.1-1~')
+    ('Stephan Sürken <absurd@debian.org>', '1.0.1-1~')
 
-    >>> cl = Changelog(mini_buildd.misc.open_utf8("./examples/doctests/changelog.oneblock.ported"), max_blocks=100)
+    >>> cl = Changelog(mini_buildd.misc.open_utf8("test-data/changelog.oneblock.ported"), max_blocks=100)
     >>> cl.find_first_not("mini-buildd@buildd.intra")
-    (u'Mini Buildd <mini-buildd@buildd.intra>', u'1.0.1-1~')
+    ('Mini Buildd <mini-buildd@buildd.intra>', '1.0.1-1~')
     """
-    def find_first_not(self, author):
-        "Find (author,version+1) of the first changelog block not by given author."
-        def s2u(string):
-            "Compat for python-debian <= 0.1.19: Author strings are of class 'str', not 'unicode'."
-            return unicode(string) if isinstance(string, str) else string
 
+    def find_first_not(self, author):
+        """Find (author,version+1) of the first changelog block not by given author."""
         result = (None, None)
         for index, block in enumerate(self._blocks):
-            result = (s2u(block.author),
+            result = (block.author,
                       "{v}".format(v=self._blocks[index + 1].version) if len(self._blocks) > (index + 1) else "{v}~".format(v=block.version))
-            if author not in s2u(block.author):
+            if author not in block.author:
                 break
         return result
 
@@ -78,20 +74,14 @@ class DebianVersion(debian.debian_support.Version):
         last_match = None
         for last_match in re.finditer(pattern, string):
             pass
-        if last_match:
-            return string[:last_match.start()] + repl + string[last_match.end():]
-        else:
-            return string + repl
+        return string[:last_match.start()] + repl + string[last_match.end():] if last_match else string + repl
 
     @classmethod
     def _get_rightmost(cls, pattern, string):
         last_match = None
         for last_match in re.finditer(pattern, string):
             pass
-        if last_match:
-            return string[last_match.start():last_match.end()]
-        else:
-            return ""
+        return string[last_match.start():last_match.end()] if last_match else ""
 
     @classmethod
     def stamp(cls):
@@ -159,33 +149,33 @@ class DebianVersion(debian.debian_support.Version):
         sid->wheezy ports:
 
         >>> DebianVersion("1.2.3-1~testSID+1").gen_internal_port(sid_regex, wheezy_default)
-        u'1.2.3-1~test70+1'
+        '1.2.3-1~test70+1'
         >>> DebianVersion("1.2.3-1~testSID+4").gen_internal_port(sid_regex, wheezy_default)
-        u'1.2.3-1~test70+4'
+        '1.2.3-1~test70+4'
         >>> DebianVersion("1.2.3-1~testSID+4fud15").gen_internal_port(sid_regex, wheezy_default)
-        u'1.2.3-1~test70+4fud15'
+        '1.2.3-1~test70+4fud15'
         >>> DebianVersion("1.2.3-1~testSID+0").gen_internal_port(sid_exp_regex, wheezy_exp_default)
-        u'1.2.3-1~test70+0'
+        '1.2.3-1~test70+0'
         >>> DebianVersion("1.2.3-1~testSID+0exp2").gen_internal_port(sid_exp_regex, wheezy_exp_default)
-        u'1.2.3-1~test70+0exp2'
+        '1.2.3-1~test70+0exp2'
 
         wheezy->squeeze ports:
 
         >>> DebianVersion("1.2.3-1~test70+1").gen_internal_port(wheezy_regex, squeeze_default)
-        u'1.2.3-1~test60+1'
+        '1.2.3-1~test60+1'
         >>> DebianVersion("1.2.3-1~test70+4").gen_internal_port(wheezy_regex, squeeze_default)
-        u'1.2.3-1~test60+4'
+        '1.2.3-1~test60+4'
         >>> DebianVersion("1.2.3-1~test70+4fud15").gen_internal_port(wheezy_regex, squeeze_default)
-        u'1.2.3-1~test60+4fud15'
+        '1.2.3-1~test60+4fud15'
         >>> DebianVersion("1.2.3-1~test70+0").gen_internal_port(wheezy_exp_regex, squeeze_exp_default)
-        u'1.2.3-1~test60+0'
+        '1.2.3-1~test60+0'
         >>> DebianVersion("1.2.3-1~test70+0exp2").gen_internal_port(wheezy_exp_regex, squeeze_exp_default)
-        u'1.2.3-1~test60+0exp2'
+        '1.2.3-1~test60+0exp2'
 
         No version restrictions: just add default version
 
         >>> DebianVersion("1.2.3-1").gen_internal_port(".*", "~port+1")
-        u'1.2.3-1~port+1'
+        '1.2.3-1~port+1'
         """
         from_apdx = self._get_rightmost(from_mandatory_version_regex, self.full_version)
         from_apdx_plus_revision = self._get_rightmost(r"\+[0-9]", from_apdx)
@@ -196,44 +186,56 @@ class DebianVersion(debian.debian_support.Version):
         return self._sub_rightmost(from_mandatory_version_regex, actual_to_default_version, self.full_version)
 
 
-class KeyringPackage(mini_buildd.misc.TmpDir):
-    def __init__(self, identity, gpg, debfullname, debemail, tpl_dir="/usr/share/doc/mini-buildd/examples/packages/archive-keyring-template"):
-        super(KeyringPackage, self).__init__()
+class TemplatePackage(mini_buildd.misc.TmpDir):
+    """Copies a package template into a temporary directory (under 'package/')."""
 
-        self.key_id = gpg.get_first_sec_key()
+    def __init__(self, template):
+        super().__init__()
+        self.template = template
+        self.package_dir = os.path.join(self.tmpdir, "package")
+        shutil.copytree(os.path.join(mini_buildd.config.PACKAGE_TEMPLATES_DIR, template), self.package_dir)
+        self.package_version = DebianVersion.stamp()
+        self.package_name = None
+
+    @property
+    def dsc(self):
+        return glob.glob(os.path.join(self.tmpdir, "*.dsc"))[0]
+
+
+class KeyringPackage(TemplatePackage):
+    def __init__(self, model):
+        super().__init__("archive-keyring")
+
+        self.key_id = model.mbd_gnupg.get_first_sec_key()
         LOG.debug("KeyringPackage using key: '{i}'".format(i=self.key_id))
 
-        self.package_name = "{i}-archive-keyring".format(i=identity)
+        self.package_name = "{i}-archive-keyring".format(i=model.identity)
         self.environment = mini_buildd.call.taint_env(
-            {"DEBEMAIL": debemail,
-             "DEBFULLNAME": debfullname,
-             "GNUPGHOME": gpg.home})
-        self.version = DebianVersion.stamp()
+            {"DEBEMAIL": model.email_address,
+             "DEBFULLNAME": model.mbd_fullname,
+             "GNUPGHOME": model.mbd_gnupg.home})
 
-        # Copy template, and replace %ID%, %MAINT% and %KEY_ID% in all files
-        p = os.path.join(self.tmpdir, "package")
-        shutil.copytree(tpl_dir, p)
-        for root, _dirs, files in os.walk(p):
+        # Replace %ID%, %MAINT% and %KEY_ID% in all files in package dir.
+        for root, _dirs, files in os.walk(self.package_dir):
             for f in files:
                 old_file = os.path.join(root, f)
                 new_file = old_file + ".new"
-                mini_buildd.misc.open_utf8(new_file, "w").write(
-                    mini_buildd.misc.subst_placeholders(
-                        mini_buildd.misc.open_utf8(old_file, "r").read(),
-                        {"ID": identity,
-                         "KEY_ID": self.key_id,
-                         "MAINT": "{n} <{e}>".format(n=debfullname, e=debemail)}))
+                with mini_buildd.misc.open_utf8(old_file, "r") as of, mini_buildd.misc.open_utf8(new_file, "w") as nf:
+                    nf.write(mini_buildd.misc.subst_placeholders(of.read(),
+                                                                 {"ID": model.identity,
+                                                                  "KEY_ID": self.key_id,
+                                                                  "MAINT": "{n} <{e}>".format(n=model.mbd_fullname, e=model.email_address)}))
                 os.rename(new_file, old_file)
 
         # Export public GnuPG key into the package
-        gpg.export(os.path.join(p, self.package_name + ".gpg"), identity=self.key_id)
+        model.mbd_gnupg.export(os.path.join(self.package_dir, self.package_name + ".gpg"), identity=self.key_id)
 
         # Generate sources.lists
         daemon = get()
         for r in mini_buildd.models.repository.Repository.objects.all():
             for d in r.distributions.all():
                 for s in r.layout.suiteoption_set.all():
-                    for rb in [None] + range(s.rollback):
+                    for rb in [None] + list(range(s.rollback)):
                         file_base = "{codename}_{archive}_{repository}_{suite}{rollback}".format(codename=d.base_source.codename,
                                                                                                  archive=daemon.model.identity,
                                                                                                  repository=r.identity,
@@ -242,15 +244,16 @@ class KeyringPackage(mini_buildd.misc.TmpDir):
                         for prefix, appendix in [("deb ", ""), ("deb-src ", "_src")]:
                             apt_line = d.mbd_get_apt_line(r, s, rollback=rb, prefix=prefix)
                             file_name = "{base}{appendix}.list".format(base=file_base, appendix=appendix)
-                            mini_buildd.misc.open_utf8(os.path.join(p, file_name), "w").write(apt_line + "\n")
+                            with mini_buildd.misc.open_utf8(os.path.join(self.package_dir, file_name), "w") as f:
+                                f.write(apt_line + "\n")
 
         # Generate changelog entry
         mini_buildd.call.Call(["debchange",
                                "--create",
                                "--package", self.package_name,
-                               "--newversion", self.version,
-                               "Automatic keyring package for archive '{i}'.".format(i=identity)],
-                              cwd=p,
+                               "--newversion", self.package_version,
+                               "Automatic keyring package for archive '{i}'.".format(i=model.identity)],
+                              cwd=self.package_dir,
                               env=self.environment).log().check()
 
         mini_buildd.call.Call(["dpkg-source",
@@ -259,31 +262,22 @@ class KeyringPackage(mini_buildd.misc.TmpDir):
                               cwd=self.tmpdir,
                               env=self.environment).log().check()
 
-        # Compute DSC file name
-        self.dsc = os.path.join(self.tmpdir,
-                                mini_buildd.changes.Changes.gen_dsc_file_name(self.package_name,
-                                                                              self.version))
 
+class TestPackage(TemplatePackage):
+    def __init__(self, template):
+        super().__init__(template)
+        self.package_name = template
 
-class DSTPackage(mini_buildd.misc.TmpDir):
-    def __init__(self, tpl_dir, version=None):
-        super(DSTPackage, self).__init__()
-
-        dst_dir = os.path.join(self.tmpdir, "package")
-        shutil.copytree(tpl_dir, dst_dir)
-        if version:
-            mini_buildd.call.Call(["debchange",
-                                   "--newversion", version,
-                                   "Version update '{v}'.".format(v=version)],
-                                  cwd=dst_dir).log().check()
+        mini_buildd.call.Call(["debchange",
+                               "--newversion", self.package_version,
+                               "Version update '{v}'.".format(v=self.package_version)],
+                              cwd=self.package_dir).log().check()
         mini_buildd.call.Call(["dpkg-source", "-b", "package"], cwd=self.tmpdir).log().check()
-        self.dsc = glob.glob(os.path.join(self.tmpdir, "*.dsc"))[0]
 
 
-class Keyrings(object):
-    """
-    Hold/manage all gnupg keyrings (for remotes and all repository uploaders).
-    """
+class Keyrings():
+    """Hold/manage all gnupg keyrings (for remotes and all repository uploaders)."""
+
     def __init__(self):
         self._our_pub_key = get().model.mbd_get_pub_key()
         self._remotes = self._gen_remotes()
@@ -295,7 +289,7 @@ class Keyrings(object):
 
     def close(self):
         self._remotes.close()
-        for u in self._uploaders.values():
+        for u in list(self._uploaders.values()):
             u.close()
 
     def _update(self):
@@ -323,7 +317,7 @@ class Keyrings(object):
         return remotes
 
     def _gen_uploaders(self):
-        "All uploader keyrings for each repository."
+        """All uploader keyrings for each repository."""
         uploaders = {}
         for r in mini_buildd.models.repository.Repository.mbd_get_active():
             uploaders[r.identity] = r.mbd_get_uploader_keyring()
@@ -334,10 +328,7 @@ class Keyrings(object):
 
 
 def run():
-    """
-    mini-buildd 'daemon engine' run.
-    """
-
+    """mini-buildd 'daemon engine' run."""
     ftpd_thread = mini_buildd.misc.run_as_thread(
         mini_buildd.ftpd.run,
         name="ftpd",
@@ -365,7 +356,7 @@ def run():
                 # Build request: builder
 
                 def queue_buildrequest(event):
-                    "Queue in extra thread so we don't block here in case builder is busy."
+                    """Queue in extra thread so we don't block here in case builder is busy."""
                     get().build_queue.put(event)
                 mini_buildd.misc.run_as_thread(queue_buildrequest, name="build queuer", daemon=True, event=event)
 
@@ -375,16 +366,16 @@ def run():
                     daemon=get(),
                     changes=changes)
 
-        except Exception as e:
-            mini_buildd.setup.log_exception(LOG, "Invalid changes file", e)
+        except BaseException as e:
+            mini_buildd.config.log_exception(LOG, "Invalid changes file", e)
 
             # Try to notify
             try:
-                subject = "INVALID CHANGES: {c}: {e}".format(c=event, e=e)
-                body = mini_buildd.misc.open_utf8(event, "r").read()
-                get().model.mbd_notify(subject, body)
-            except Exception as e:
-                mini_buildd.setup.log_exception(LOG, "Invalid changes notify failed", e)
+                with mini_buildd.misc.open_utf8(event, "r") as body:
+                    subject = "INVALID CHANGES: {c}: {e}".format(c=event, e=e)
+                    get().model.mbd_notify(subject, body.read())
+            except BaseException as e:
+                mini_buildd.config.log_exception(LOG, "Invalid changes notify failed", e)
 
             # Try to clean up
             try:
@@ -392,8 +383,8 @@ def run():
                     changes.remove()
                 else:
                     os.remove(event)
-            except Exception as e:
-                mini_buildd.setup.log_exception(LOG, "Invalid changes cleanup failed", e)
+            except BaseException as e:
+                mini_buildd.config.log_exception(LOG, "Invalid changes cleanup failed", e)
 
         finally:
             get().incoming_queue.task_done()
@@ -407,11 +398,11 @@ def run():
     # There should be a nicer way, really...
     try:
         get().keyrings.close()
-    except:
+    except BaseException:
         pass
 
 
-class Daemon(object):
+class Daemon():
     def __init__(self):
         # Set global to ourself
         global _INSTANCE  # pylint: disable=global-statement
@@ -432,7 +423,7 @@ class Daemon(object):
         self.last_packages = None
         self.last_builds = None
 
-    def __unicode__(self):
+    def __str__(self):
         return "{r}: {d}".format(r="UP" if self.is_running() else "DOWN", d=self.model)
 
     @classmethod
@@ -456,7 +447,7 @@ class Daemon(object):
             self.keyrings = Keyrings()
         else:
             self.keyrings.set_needs_update()
-        self.incoming_queue = Queue.Queue()
+        self.incoming_queue = queue.Queue()
         self.build_queue = mini_buildd.misc.BlockQueue(maxsize=self.model.build_queue_size)
         self.packages = {}
         self.builds = {}
@@ -471,17 +462,18 @@ class Daemon(object):
                 if p.api_check():
                     self.last_packages.append(p)
                 else:
-                    LOG.warn("Removing (new API) from last package info: {p}".format(p=p))
+                    LOG.warning("Removing (new API) from last package info: {p}".format(p=p))
             for b in last_builds:
                 if b.api_check():
                     self.last_builds.append(b)
                 else:
-                    LOG.warn("Removing (new API) from last builds info: {b}".format(b=b))
-        except Exception as e:
-            mini_buildd.setup.log_exception(LOG, "Error adding persisted last builds/packages (ignoring)", e, logging.WARN)
+                    LOG.warning("Removing (new API) from last builds info: {b}".format(b=b))
+        except BaseException as e:
+            mini_buildd.config.log_exception(LOG, "Error adding persisted last builds/packages (ignoring)", e, logging.WARN)
 
-    def update_to_model(self, obj):
-        obj.mbd_set_pickled_data((self.last_packages, self.last_builds))
+    def update_to_model(self):
+        # Save pickled persistent state
+        self.model.mbd_set_pickled_data((self.last_packages, self.last_builds))
 
     def start(self, force_check=False, msglog=LOG):
         with self.lock:
@@ -493,7 +485,7 @@ class Daemon(object):
                     self.thread = mini_buildd.misc.run_as_thread(run, name="packager")
                     msglog.info("Daemon started.")
                 else:
-                    msglog.warn("Daemon is deactivated (won't start). Please (re)configure your instance as superuser.")
+                    msglog.warning("Daemon is deactivated (won't start). Please (re)configure your instance as superuser.")
             else:
                 msglog.info("Daemon already running.")
 
@@ -502,11 +494,9 @@ class Daemon(object):
     def stop(self, msglog=LOG):
         with self.lock:
             if self.thread:
-                # Save pickled persistend state; as a workaround, save the whole model but on fresh object/db state.
-                # With django 1.5, we could just use save(update_fields=["pickled_data"]) on self.model
-                model = self._new_model_object()
-                self.update_to_model(model)
-                model.save()
+                # Save pickled persistend state
+                self.update_to_model()
+                self.model.save(update_fields=["pickled_data"])
 
                 self.incoming_queue.put("SHUTDOWN")
                 self.thread.join()
@@ -518,6 +508,10 @@ class Daemon(object):
 
         return not self.is_running()
 
+    def restart(self, force_check=False, msglog=LOG):
+        self.stop(msglog=msglog)
+        return self.start(force_check=force_check, msglog=msglog)
+
     def is_busy(self):
         return self.lock.locked()
 
@@ -525,12 +519,12 @@ class Daemon(object):
         return not self.is_busy() and bool(self.thread)
 
     def get_status(self):
-        status = mini_buildd.api.Status([])
-        status.run(self)
+        status = mini_buildd.api.Status({}, daemon=self)
+        status.run()
         return status
 
     def get_title(self):
-        "Human-readable short title for this Daemon instance."
+        """Human-readable short title for this Daemon instance."""
         return "{id}@{hn}".format(id=self.model.identity, hn=self.model.hostname)
 
     @classmethod
@@ -540,8 +534,14 @@ class Daemon(object):
 
     @classmethod
     def logcat(cls, lines):
-        logfile = mini_buildd.misc.open_utf8(mini_buildd.setup.LOG_FILE, "r")
-        return mini_buildd.misc.tail(logfile, lines)
+        with mini_buildd.misc.open_utf8(mini_buildd.config.LOG_FILE) as lf:
+            return "".join(collections.deque(lf, lines))
+
+    def get_last_packages(self):
+        return sorted({p.changes["source"] for p in self.last_packages})
+
+    def get_last_versions(self, package):
+        return sorted({p.changes["version"] for p in self.last_packages if p.changes["source"] == package})
 
     @classmethod
     def get_active_chroots(cls):
@@ -551,6 +551,19 @@ class Daemon(object):
     def get_active_repositories(cls):
         return mini_buildd.models.repository.Repository.mbd_get_active()
 
+    @classmethod
+    def get_suites(cls):
+        return mini_buildd.models.repository.Suite.objects.all()
+
+    @classmethod
+    def get_active_codenames(cls):
+        codenames = []
+        for r in cls.get_active_repositories():
+            for d in r.distributions.all():
+                if d.base_source.codename not in codenames:
+                    codenames.append(d.base_source.codename)
+        return codenames
+
     @classmethod
     def get_active_remotes(cls):
         return mini_buildd.models.gnupg.Remote.mbd_get_active()
@@ -565,9 +578,7 @@ class Daemon(object):
 
     @classmethod
     def parse_distribution(cls, dist):
-        """
-        Get repository, distribution and suite model objects (plus rollback no) from distribtion string.
-        """
+        """Get repository, distribution and suite model objects (plus rollback no) from distribution string."""
         # Check and parse changes distribution string
         dist_parsed = mini_buildd.misc.Distribution(mini_buildd.models.repository.map_incoming_distribution(dist))
 
@@ -582,7 +593,7 @@ class Daemon(object):
 
     def _port(self, dsc_url, package, dist, version, comments=None, options=None):
         def changelog_entries():
-            "Merge comments and options into plain changelog_entries."
+            """Merge comments and options into plain changelog_entries."""
             changelog_entries = comments or []
             for o in options or self._DEFAULT_PORT_OPTIONS:
                 changelog_entries.append("{keyword}: {option}".format(keyword=mini_buildd.changes.Changes.Options.KEYWORD, option=o))
@@ -617,8 +628,8 @@ class Daemon(object):
             dst_path = os.path.join(t.tmpdir, dst)
 
             # Get version and author from original changelog; use the first block not
-            original_author, original_version = Changelog(mini_buildd.misc.open_utf8(os.path.join(dst_path, "debian", "changelog"), "r"),
-                                                          max_blocks=100).find_first_not(self.model.email_address)
+            with mini_buildd.misc.open_utf8(os.path.join(dst_path, "debian", "changelog"), "r") as cl:
+                original_author, original_version = Changelog(cl, max_blocks=100).find_first_not(self.model.email_address)
             LOG.debug("Port: Found original version/author: {v}/{a}".format(v=original_version, a=original_author))
 
             # Change changelog in DST
@@ -654,12 +665,12 @@ class Daemon(object):
 
             # Generate Changes file
             with open(changes, "w+") as out:
-                # Note: dpkg-genchanges has a home-brewed options parser. It does not allow, for example, "-v 1.2.3", only "-v1.2.3", so we need to use *one* sequence item fo that.
+                # Note: dpkg-genchanges has a home-brewed options parser. It does not allow, for example, "-v 1.2.3", only "-v1.2.3", so we need to use *one* sequence item for that.
                 mini_buildd.call.Call(["dpkg-genchanges",
                                        "-S",
                                        "-sa",
                                        "-v{v}".format(v=original_version),
-                                       "-DX-Mini-Buildd-Originally-Changed-By={a}".format(a=original_author).encode(mini_buildd.setup.CHAR_ENCODING)],
+                                       "-DX-Mini-Buildd-Originally-Changed-By={a}".format(a=original_author)],
                                       cwd=dst_path,
                                       env=env,
                                       stdout=out).log().check()
@@ -667,7 +678,8 @@ class Daemon(object):
             # Sign and add to incoming queue
             self.model.mbd_gnupg.sign(changes)
             self.incoming_queue.put(changes)
-        except:
+            return dist, package, version
+        except BaseException:
             t.close()
             raise
 
@@ -676,7 +688,7 @@ class Daemon(object):
         from_repository, from_distribution, from_suite, _from_rollback = self.parse_distribution(from_dist)
         p = from_repository.mbd_package_find(package, distribution=from_dist, version=version)
         if not p:
-            raise Exception("Port failed: Package (version) for '{p}' not found in '{d}'".format(p=package, d=from_dist))
+            raise Exception("Port failed: Package '{p}' ({v}) not found in '{d}'".format(p=package, v=version, d=from_dist))
 
         # check to_dist
         to_repository, to_distribution, to_suite, to_rollback = self.parse_distribution(to_dist)
@@ -693,11 +705,11 @@ class Daemon(object):
             port_version = v.gen_internal_port(from_repository.layout.mbd_get_mandatory_version_regex(from_repository, from_distribution, from_suite),
                                                to_repository.layout.mbd_get_default_version(to_repository, to_distribution, to_suite))
 
-        _component, url = from_repository.mbd_get_dsc_url(from_distribution, package, p["sourceversion"])
-        if not url:
+        _component, path = from_repository.mbd_get_dsc_path(from_distribution, package, p["sourceversion"])
+        if not path:
             raise Exception("Port failed: Can't find DSC for {p}-{v} in pool".format(p=package, v=p["sourceversion"]))
 
-        self._port(url, package, to_dist, port_version, options=options)
+        return self._port(urllib.parse.urljoin(self.model.mbd_get_http_url(), path), package, to_dist, port_version, options=options)
 
     def portext(self, dsc_url, to_dist, options=None):
         # check to_dist
@@ -709,25 +721,14 @@ class Daemon(object):
         if to_rollback:
             raise Exception("Port failed: Rollback distribution requested: '{d}'".format(d=to_dist))
 
-        dsc = debian.deb822.Dsc(urllib2.urlopen(dsc_url))
+        dsc = debian.deb822.Dsc(mini_buildd.net.urlopen_ca_certificates(dsc_url))
         v = DebianVersion(dsc["Version"])
-        self._port(dsc_url,
-                   dsc["Source"],
-                   to_dist,
-                   v.gen_external_port(to_repository.layout.mbd_get_default_version(to_repository, to_distribution, to_suite)),
-                   comments=["External port from: {u}".format(u=dsc_url)],
-                   options=options)
-
-    def get_keyring_package(self):
-        return KeyringPackage(self.model.identity,
-                              self.model.mbd_gnupg,
-                              self.model.mbd_fullname,
-                              self.model.email_address)
-
-    @classmethod
-    def get_test_package(cls, id_):
-        return DSTPackage("/usr/share/doc/mini-buildd/examples/packages/mbd-test-{i}".format(i=id_),
-                          version=DebianVersion.stamp())
+        return self._port(dsc_url,
+                          dsc["Source"],
+                          to_dist,
+                          v.gen_external_port(to_repository.layout.mbd_get_default_version(to_repository, to_distribution, to_suite)),
+                          comments=["External port from: {u}".format(u=dsc_url)],
+                          options=options)
 
     def mbd_get_sources_list(self, codename, repo_regex, suite_regex, prefixes, with_extra_sources):
         apt_lines = []
