@@ -2,8 +2,6 @@ import os
 import logging
 import random
 
-import tzlocal
-
 import django
 import django.conf
 
@@ -123,12 +121,12 @@ def configure(smtp_string, loglevel):
         DATABASES={"default": {"ENGINE": "django.db.backends.sqlite3",
                                "NAME": os.path.join(mini_buildd.config.HOME_DIR, "config.sqlite")}},
 
-        TIME_ZONE=tzlocal.get_localzone().zone,
-        USE_TZ=True,  # Django: "stores datetime information in UTC in the database": that's definitely what we want
+        TIME_ZONE="Etc/UTC",  # Default time zone: Canonical UTC. Better than django's default 'America/Chicago'.
+        USE_TZ=True,          # Django: "stores datetime information in UTC in the database": that's definitely what we want.
         USE_L10N=True,
 
         SECRET_KEY=get_django_secret_key(mini_buildd.config.HOME_DIR),
-        ROOT_URLCONF="mini_buildd.root_urls",
+        ROOT_URLCONF="mini_buildd.urls",
         STATIC_URL="/static/",
         ACCOUNT_ACTIVATION_DAYS=3,
         LOGIN_REDIRECT_URL="/mini_buildd/",
