@@ -1,30 +1,28 @@
-# -*- coding: utf-8 -*-
-from __future__ import unicode_literals
-from __future__ import absolute_import
-
 import os
 import re
 import tempfile
 import shutil
+import glob
 import logging
 
 import mini_buildd.misc
 import mini_buildd.call
-import mini_buildd.setup
+import mini_buildd.config
 
 LOG = logging.getLogger(__name__)
 
 
-class Colons(object):
+class Colons():
     """
     Provide a colon->name mapping for the gpg script-parsable '--with-colons' output.
 
     See /usr/share/doc/gnupg/DETAILS.gz.
     """
+
     def __init__(self, colons_line):
         self._colons = colons_line.split(":")
 
-    def __unicode__(self):
+    def __str__(self):
         return "{t}: {k}: {u}".format(t=self.type, k=self.key_id, u=self.user_id)
 
     def _get(self, index):
@@ -48,33 +46,35 @@ class Colons(object):
 
     @property
     def user_id(self):
-        "fingerprint for 'fpr' type"
+        """Fingerprint for 'fpr' type."""
         return self._get(9)
 
 
-class BaseGnuPG(object):
+class BaseGnuPG():
     @classmethod
     def get_flavor(cls):
         """
-        Ugly-parse GPG binary flavor(=major.minor) "1.4"
-        ("classic"), "2.0" ("stable") or "2.1" ("modern") from
-        "gpg --version" output (like "gpg (GnuPG)
-        2.1.14"). Don't fail but return "unknown" if anything
-        nasty happens.
+        Ugly-parse GPG binary flavor(=major.minor).
+
+        "1.4" ("classic"), "2.0" ("stable") or "2.1" ("modern") from
+        "gpg --version" output (like "gpg (GnuPG) 2.1.14"). Don't fail
+        but return "unknown" if anything nasty happens.
+
         """
         try:
-            version_info = mini_buildd.call.Call(["gpg", "--version"]).check().ustdout.splitlines()
+            version_info = mini_buildd.call.Call(["gpg", "--version"]).check().stdout.splitlines()
             version_line = version_info[0].split(" ")
             return version_line[2][0:3]
-        except:
-            return "unkown"
+        except BaseException as e:
+            LOG.warning("Can't parse GPG flavor: {e}".format(e=e))
+            return "unknown"
 
     def __init__(self, home):
         self.flavor = self.get_flavor()
         self.home = home
         self.gpg_cmd = ["gpg",
                         "--homedir", home,
-                        "--display-charset", mini_buildd.setup.CHAR_ENCODING,
+                        "--display-charset", mini_buildd.config.CHAR_ENCODING,
                         "--batch"]
         LOG.info("GPG {f}: {c}".format(f=self.flavor, c=self.gpg_cmd))
 
@@ -83,8 +83,8 @@ class BaseGnuPG(object):
                             "2.2": "\n%no-protection\n"}
 
         with tempfile.TemporaryFile() as t:
-            t.write(template.encode(mini_buildd.setup.CHAR_ENCODING))
-            t.write(flavor_additions.get(self.flavor, ""))
+            t.write(template.encode(mini_buildd.config.CHAR_ENCODING))
+            t.write(flavor_additions.get(self.flavor, "").encode(mini_buildd.config.CHAR_ENCODING))
             t.seek(0)
             mini_buildd.call.Call(self.gpg_cmd + ["--gen-key"], stdin=t).log().check()
 
@@ -93,10 +93,10 @@ class BaseGnuPG(object):
             mini_buildd.call.Call(self.gpg_cmd + ["--export"] + ([identity] if identity else []), stdout=f).check()
 
     def get_pub_key(self, identity):
-        return mini_buildd.call.Call(self.gpg_cmd + ["--armor", "--export", identity]).log().check().ustdout
+        return mini_buildd.call.Call(self.gpg_cmd + ["--armor", "--export", identity]).log().check().stdout
 
     def _get_colons(self, list_arg="--list-public-keys", type_regex=".*"):
-        for line in mini_buildd.call.Call(self.gpg_cmd + [list_arg, "--with-colons", "--fixed-list-mode", "--with-fingerprint", "--with-fingerprint"]).log().check().ustdout.splitlines():
+        for line in mini_buildd.call.Call(self.gpg_cmd + [list_arg, "--with-colons", "--fixed-list-mode", "--with-fingerprint", "--with-fingerprint"]).log().check().stdout.splitlines():
             colons = Colons(line)
             LOG.debug("{c}".format(c=colons))
             if re.match(type_regex, colons.type):
@@ -110,7 +110,7 @@ class BaseGnuPG(object):
 
     def get_first_sec_colon(self, type_regex):
         try:
-            return self.get_sec_colons(type_regex=type_regex).next()
+            return next(self.get_sec_colons(type_regex=type_regex))
         except StopIteration:
             return Colons("")
 
@@ -123,12 +123,25 @@ class BaseGnuPG(object):
     def get_first_sec_key_user_id(self):
         return self.get_first_sec_colon("uid").user_id
 
-    def recv_key(self, keyserver, identity):
-        return mini_buildd.call.Call(self.gpg_cmd + ["--armor", "--keyserver", keyserver, "--recv-keys", identity]).log().check().ustdout
+    def import_pub_key(self, key_server, identity):
+        # 1st, try keyrings on local system
+        for keyring in glob.glob("/usr/share/keyrings/*.gpg"):
+            try:
+                # Note that gpg --export succeeds even if nothing can be exported, --import however fails on empty input
+                key = mini_buildd.call.Call(self.gpg_cmd + ["--armor", "--keyring", keyring, "--export", identity]).log().check().stdout
+                mini_buildd.call.Call(self.gpg_cmd + ["--armor", "--import", "-"], input=bytes(key, encoding="ascii")).log().check()
+                LOG.info("{id}: Imported from keyring: {k}".format(id=identity, k=keyring))
+                return
+            except BaseException as e:
+                LOG.info("{id}: Can't import from keyring {k}: {e}".format(id=identity, k=keyring, e=e))
+
+        # 2nd, try key_server (with retry)
+        mini_buildd.call.call_with_retry(self.gpg_cmd + ["--armor", "--keyserver", key_server, "--recv-keys", identity], retry_max_tries=5, retry_sleep=5)
+        LOG.info("{id}: Imported from key server: {k}".format(id=identity, k=key_server))
 
     def add_pub_key(self, key):
         with tempfile.TemporaryFile() as t:
-            t.write(key.encode(mini_buildd.setup.CHAR_ENCODING))
+            t.write(key.encode(mini_buildd.config.CHAR_ENCODING))
             t.seek(0)
             mini_buildd.call.Call(self.gpg_cmd + ["--import"], stdin=t).log().check()
 
@@ -136,12 +149,12 @@ class BaseGnuPG(object):
         if os.path.exists(keyring):
             self.gpg_cmd += ["--keyring", keyring]
         else:
-            LOG.warn("Skipping non-existing keyring file: {k}".format(k=keyring))
+            LOG.warning("Skipping non-existing keyring file: {k}".format(k=keyring))
 
     def verify(self, signature, data=None):
         try:
             mini_buildd.call.Call(self.gpg_cmd + ["--verify", signature] + ([data] if data else [])).check()
-        except:
+        except BaseException:
             raise Exception("GnuPG authorization failed.")
 
     def sign(self, file_name, identity=None):
@@ -160,9 +173,9 @@ class BaseGnuPG(object):
                 os.remove(signed_file)
 
         # Retrying sign call; workaround for mystery https://bugs.debian.org/cgi-bin/bugreport.cgi?bug=849551
-        mini_buildd.call.call_with_retry(self.gpg_cmd +
-                                         ["--armor", "--textmode", "--clearsign", "--output", signed_file] +
-                                         (["--local-user", identity] if identity else []) + [unsigned_file],
+        mini_buildd.call.call_with_retry(self.gpg_cmd
+                                         + ["--armor", "--textmode", "--clearsign", "--output", signed_file]
+                                         + (["--local-user", identity] if identity else []) + [unsigned_file],
                                          retry_max_tries=5,
                                          retry_sleep=1,
                                          retry_failed_cleanup=failed_cleanup)
@@ -174,7 +187,7 @@ class BaseGnuPG(object):
 
 class GnuPG(BaseGnuPG):
     def __init__(self, template, fullname, email):
-        super(GnuPG, self).__init__(home=os.path.join(mini_buildd.setup.HOME_DIR, ".gnupg"))
+        super().__init__(home=os.path.join(mini_buildd.config.HOME_DIR, ".gnupg"))
         self.template = """\
 {t}
 Name-Real: {n}
@@ -195,31 +208,34 @@ Name-Email: {e}
             LOG.info("GnuPG setup removed: {h}".format(h=self.home))
 
     def get_pub_key(self, identity=None):
-        return super(GnuPG, self).get_pub_key("mini-buildd")
+        return super().get_pub_key("mini-buildd")
 
 
 class TmpGnuPG(BaseGnuPG, mini_buildd.misc.TmpDir):
-    """
-    >>> # mini_buildd.setup.DEBUG.append("keep")  # Enable 'keep' for debugging only
+    r"""
+    Temporary GnuPG.
+
+    >>> # mini_buildd.config.DEBUG.append("keep")  # Enable 'keep' for debugging only
     >>> gnupg_home = mini_buildd.misc.TmpDir()
-    >>> shutil.copy2("./examples/doctests/gpg/secring.gpg", gnupg_home.tmpdir)
-    >>> shutil.copy2("./examples/doctests/gpg/pubring.gpg", gnupg_home.tmpdir)
+    >>> dummy = shutil.copy2("test-data/gpg/secring.gpg", gnupg_home.tmpdir)
+    >>> dummy = shutil.copy2("test-data/gpg/pubring.gpg", gnupg_home.tmpdir)
     >>> gnupg = BaseGnuPG(home=gnupg_home.tmpdir)
 
     >>> gnupg.get_first_sec_colon("sec").type
-    u'sec'
+    'sec'
     >>> gnupg.get_first_sec_key_user_id()
-    u'\\xdcdo \\xdcmlaut <test@key.org>'
+    'Üdo Ümlaut <test@key.org>'
     >>> gnupg.get_first_sec_key()  #doctest: +ELLIPSIS
-    u'AF95FC80FC40A82E'
+    'AF95FC80FC40A82E'
     >>> gnupg.get_first_sec_key_fingerprint()  #doctest: +ELLIPSIS
-    u'4FB13BDD777C046D72D4E7D3AF95FC80FC40A82E'
+    '4FB13BDD777C046D72D4E7D3AF95FC80FC40A82E'
 
     >>> export = tempfile.NamedTemporaryFile()
     >>> gnupg.export(export.name)
 
     >>> t = tempfile.NamedTemporaryFile()
-    >>> t.write("A test file\\n")
+    >>> t.write(b"A test file\n")
+    12
     >>> t.flush()
     >>> gnupg.sign(file_name=t.name, identity="test@key.org")
     >>> gnupg.verify(t.name)
@@ -231,6 +247,7 @@ class TmpGnuPG(BaseGnuPG, mini_buildd.misc.TmpDir):
     >>> tgnupg.close()
     >>> gnupg_home.close()
     """
+
     def __init__(self):
         mini_buildd.misc.TmpDir.__init__(self)
-        super(TmpGnuPG, self).__init__(home=self.tmpdir)
+        super().__init__(home=self.tmpdir)
