@@ -1,17 +1,12 @@
-# -*- coding: utf-8 -*-
 """
 Django modules package.
 
 All models must provide a an admin meta class as 'Model.Admin'.
 """
-from __future__ import unicode_literals
-from __future__ import absolute_import
 
 
 def import_all():
-    """
-    Call this after your django app is configured.
-    """
+    """Call this after your django app is configured."""
     import django.contrib
 
     from mini_buildd.models import gnupg
