@@ -1,5 +1,4 @@
-# -*- coding: utf-8 -*-
-r"""
+"""
 Generic module for models of the django app *mini_buildd*.
 
 Naming conventions
@@ -8,7 +7,7 @@ Naming conventions
 Model class and field names
 ---------------------------
 All model class names and all field names must be **human
-readable with no abbrevations** (as django, per default,
+readable with no abbreviations** (as django, per default,
 displays the internal names intelligently to the end user).
 
 *Model class names* must be in **CamelCase**.
@@ -29,14 +28,12 @@ To group fields together for the end user, use AdminModel's *fieldset* option.
 
 Methods
 -------
+Any methods that represent mini-buildd logic should go into the models
+directly, but must be prefixed with "mbd_". This avoids conflicts
+with method names form the django model's class, but still keeps the
+logic where it belongs.
 
-Methods that represent mini-buildd logic should go into the
-models directly, but must be prefixed with "mbd\_". This avoids
-conflicts with method names form the django model's class, but
-still keeps the logic where it belongs.
 """
-from __future__ import unicode_literals
-from __future__ import absolute_import
 
 import datetime
 import re
@@ -44,8 +41,6 @@ import pickle
 import base64
 import logging
 
-import pytz
-
 import django.db.models
 import django.contrib.messages
 import django.contrib.admin
@@ -55,7 +50,7 @@ import django.core.exceptions
 import django.template.response
 import django.utils.timezone
 
-import mini_buildd.setup
+import mini_buildd.config
 
 from mini_buildd.models.msglog import MsgLog
 LOG = logging.getLogger(__name__)
@@ -70,16 +65,18 @@ LOG = logging.getLogger(__name__)
 # single objects from the model's form.
 try:
     django.contrib.admin.site.disable_action("delete_selected")
-except:
-    pass
+except BaseException as e:
+    LOG.warning("Error disabling delete action (ignoring): {e}".format(e=e))
 
 
 class Model(django.db.models.Model):
-    """Abstract father model for all mini-buildd models.
+    """
+    Abstract father model for all mini-buildd models.
 
     This just makes sure no config is changed or deleted while
     the daemon is running.
     """
+
     extra_options = django.db.models.TextField(blank=True, editable=True,
                                                help_text="""\
 Extra/experimental options (in the form 'KEY: VALUE' per line) a
@@ -97,27 +94,25 @@ are actually supported by the current model.
     # May be used by any model for persistent python state
     pickled_data = django.db.models.TextField(blank=True, editable=False)
 
-    class Meta(object):
+    class Meta():
         abstract = True
         app_label = "mini_buildd"
 
     class Admin(django.contrib.admin.ModelAdmin):
         @classmethod
         def _mbd_on_change(cls, request, obj):
-            "Global actions to take when an object changes."
+            """Global actions to take when an object changes."""
             for o in obj.mbd_get_reverse_dependencies():
                 o.mbd_set_changed(request)
                 o.save()
 
         @classmethod
         def _mbd_on_activation(cls, request, obj):
-            "Global actions to take when an object becomes active."
-            pass
+            """Global actions to take when an object becomes active."""
 
         @classmethod
         def _mbd_on_deactivation(cls, request, obj):
-            "Global actions to take when an object becomes inactive."
-            pass
+            """Global actions to take when an object becomes inactive."""
 
         def save_model(self, request, obj, form, change):
             if change:
@@ -136,8 +131,8 @@ are actually supported by the current model.
 
     @classmethod
     def mbd_get_daemon(cls):
-        import mini_buildd.daemon
-        return mini_buildd.daemon.get()
+        from mini_buildd.daemon import get
+        return get()
 
     def mbd_get_extra_options(self):
         result = {}
@@ -152,18 +147,13 @@ are actually supported by the current model.
 
     def mbd_get_pickled_data(self, default=None):
         try:
-            return pickle.loads(base64.decodestring(self.pickled_data))
-        except Exception as e:
-            mini_buildd.setup.log_exception(LOG, "Ignoring unpickling error", e)
-            try:
-                # Be nice and also try the olde way <1.0 beta.7
-                return pickle.loads(self.pickled_data.encode(mini_buildd.setup.CHAR_ENCODING))
-            except Exception as e:
-                mini_buildd.setup.log_exception(LOG, "Ignoring compat unpickling error", e)
-                return default
+            return pickle.loads(base64.decodebytes(bytes(self.pickled_data, encoding=mini_buildd.config.CHAR_ENCODING)))
+        except BaseException as e:
+            mini_buildd.config.log_exception(LOG, "Ignoring unpickling error", e)
+            return default
 
     def mbd_set_pickled_data_pickled(self, pickled_data):
-        self.pickled_data = base64.encodestring(pickled_data)
+        self.pickled_data = str(base64.encodebytes(pickled_data), encoding=mini_buildd.config.CHAR_ENCODING)
 
     def mbd_set_pickled_data(self, data):
         self.mbd_set_pickled_data_pickled(pickle.dumps(data, pickle.HIGHEST_PROTOCOL))
@@ -183,7 +173,7 @@ are actually supported by the current model.
 
     @classmethod
     def mbd_get_or_create(cls, msglog, **kwargs):
-        "Like get_or_create, but adds a info message."
+        """Like get_or_create, but adds a info message."""
         obj, created = cls.objects.get_or_create(**kwargs)
         if created:
             msglog.info("Created: {o}".format(o=obj))
@@ -193,9 +183,8 @@ are actually supported by the current model.
 
 
 class StatusModel(Model):
-    """
-    Abstract model class for all models that carry a status. See Manual: :ref:`admin_configuration`.
-    """
+    """Abstract model class for all models that carry a status. See Manual: :ref:`admin_configuration`."""
+
     # The main statuses: removed, prepared, active
     STATUS_REMOVED = 0
     STATUS_PREPARED = 1
@@ -212,10 +201,10 @@ class StatusModel(Model):
 
     # Statuses of the prepared data, relevant for status "Prepared" only.
     # For "Removed" it's always NONE, for "Active" it's always the stamp of the last check.
-    CHECK_NONE = datetime.datetime(datetime.MINYEAR, 1, 1, tzinfo=pytz.utc)
-    CHECK_CHANGED = datetime.datetime(datetime.MINYEAR, 1, 2, tzinfo=pytz.utc)
-    CHECK_FAILED = datetime.datetime(datetime.MINYEAR, 1, 3, tzinfo=pytz.utc)
-    CHECK_REACTIVATE = datetime.datetime(datetime.MINYEAR, 1, 4, tzinfo=pytz.utc)
+    CHECK_NONE = datetime.datetime(datetime.MINYEAR, 1, 1, tzinfo=datetime.timezone.utc)
+    CHECK_CHANGED = datetime.datetime(datetime.MINYEAR, 1, 2, tzinfo=datetime.timezone.utc)
+    CHECK_FAILED = datetime.datetime(datetime.MINYEAR, 1, 3, tzinfo=datetime.timezone.utc)
+    CHECK_REACTIVATE = datetime.datetime(datetime.MINYEAR, 1, 4, tzinfo=datetime.timezone.utc)
     _CHECK_MAX = CHECK_REACTIVATE
     CHECK_STRINGS = {
         CHECK_NONE: {"char": "-", "string": "Unchecked -- please run check"},
@@ -233,26 +222,28 @@ class StatusModel(Model):
         abstract = True
 
     class Admin(Model.Admin):
-        def save_model(self, request, obj, form, change):  # pylint: disable=super-on-old-class
+        def save_model(self, request, obj, form, change):
             if change:
                 obj.mbd_set_changed(request)
-            super(StatusModel.Admin, self).save_model(request, obj, form, change)
+            super().save_model(request, obj, form, change)
 
         @classmethod
         def _mbd_run_dependencies(cls, request, obj, func, **kwargs):
             """
-            Run action for all dependencies, but don't fail and run all checks for models with
-            LETHAL_DEPENDENCIES set to False. Practical use case is the Daemon model
-            only, where we want to run all checks on all dependencies, but not fail ourselves.
+            Run action for all dependencies.
+
+            But don't fail and run all checks for models with
+            LETHAL_DEPENDENCIES set to False. Practical use case is
+            the Daemon model only, where we want to run all checks on
+            all dependencies, but not fail ourselves.
             """
             for o in obj.mbd_get_dependencies():
                 try:
                     func(request, o, **kwargs)
-                except Exception as e:
+                except BaseException as e:
                     if obj.LETHAL_DEPENDENCIES:
                         raise
-                    else:
-                        MsgLog(LOG, request).warn("Check on '{o}' failed: {e}".format(o=o, e=e))
+                    MsgLog(LOG, request).warning("Check on '{o}' failed: {e}".format(o=o, e=e))
 
         @classmethod
         def mbd_prepare(cls, request, obj):
@@ -308,7 +299,7 @@ class StatusModel(Model):
 
                     if needs_activation and not obj.mbd_is_active():
                         raise Exception("Not active, but a (tobe-)active item depends on it. Activate this first: {o}".format(o=obj))
-                except:
+                except BaseException:
                     # Check failed, auto-deactivate and re-raise exception
                     obj.last_checked = max(obj.last_checked, obj.CHECK_FAILED)
                     if obj.mbd_is_active():
@@ -358,14 +349,15 @@ class StatusModel(Model):
         @classmethod
         def mbd_action(cls, request, queryset, action, **kwargs):
             """
-            Try to run action on each object in queryset, emit
-            error message on failure, but don't fail ourself.
+            Try to run action on each object in queryset.
+
+            Emit error message on failure, but don't fail ourself.
             """
             for o in queryset:
                 try:
                     getattr(cls, "mbd_" + action)(request, o, **kwargs)
-                except Exception as e:
-                    mini_buildd.setup.log_exception(MsgLog(LOG, request), "{a} failed: {o}".format(a=action, o=o), e)
+                except BaseException as e:
+                    mini_buildd.config.log_exception(MsgLog(LOG, request), "{a} failed: {o}".format(a=action, o=o), e)
 
         def mbd_action_prepare(self, request, queryset):
             self.mbd_action(request, queryset, "prepare")
@@ -383,7 +375,7 @@ class StatusModel(Model):
             self.mbd_action(request, queryset, "deactivate")
         mbd_action_deactivate.short_description = "Deactivate"
 
-        def mbd_action_remove(self, request, queryset):
+        def mbd_action_remove(self, request, queryset):  # pylint: disable=inconsistent-return-statements
             if request.POST.get("confirm"):
                 self.mbd_action(request, queryset, "remove")
             else:
@@ -414,23 +406,22 @@ this would mean losing all packages!
 
         @classmethod
         def mbd_meta_pca_all(cls, msglog):
-            "Run prepare, check, and activate for all objects of this model"
+            """Run prepare, check, and activate for all objects of this model."""
             cls.mbd_action(msglog.request, cls.mbd_model.objects.all(), "prepare")
             cls.mbd_action(msglog.request, cls.mbd_model.objects.all(), "check")
             cls.mbd_action(msglog.request, cls.mbd_model.objects.all(), "activate")
 
         def colored_status(self, obj):  # pylint: disable=no-self-use
-            return '<div style="font-weight:bold;background-color:{bc};color:{fc};padding:2px 0px 2px 5px" title="{t}">{o}</div>'.format(
+            return django.utils.html.format_html(
+                '<div style="font-weight:bold;background-color:{bc};color:{fc};padding:2px 0px 2px 5px" title="{t}">{o}</div>',
                 bc=obj.STATUS_COLORS[obj.status].get("bg"),
                 fc=obj.STATUS_COLORS[obj.status].get("fg"),
                 t=obj.mbd_get_status_display(typ="string"),
                 o=obj.mbd_get_status_display(typ="char"))
 
-        colored_status.allow_tags = True
-
         actions = [mbd_action_prepare, mbd_action_check, mbd_action_pc, mbd_action_activate, mbd_action_pca, mbd_action_deactivate, mbd_action_remove]
-        list_display = ["colored_status", "__unicode__"]
-        list_display_links = ["__unicode__"]
+        list_display = ["colored_status", "__str__"]
+        list_display_links = ["__str__"]
 
     @property
     def days_until_recheck(self):
@@ -445,9 +436,9 @@ this would mean losing all packages!
     def mbd_set_changed(self, request):
         if self.mbd_is_active():
             self.status = self.STATUS_PREPARED
-            MsgLog(LOG, request).warn("Deactivated due to changes: {o}".format(o=self))
+            MsgLog(LOG, request).warning("Deactivated due to changes: {o}".format(o=self))
         self.last_checked = self.CHECK_CHANGED
-        MsgLog(LOG, request).warn("Marked as changed: {o}".format(o=self))
+        MsgLog(LOG, request).warning("Marked as changed: {o}".format(o=self))
 
     #
     # Action hooks helpers
@@ -480,19 +471,14 @@ this would mean losing all packages!
 
     @classmethod
     def mbd_get_active_or_auto_reactivate(cls):
-        return cls.objects.filter(django.db.models.Q(status__gte=cls.STATUS_ACTIVE) |
-                                  django.db.models.Q(last_checked=cls.CHECK_REACTIVATE))
+        return cls.objects.filter(django.db.models.Q(status__gte=cls.STATUS_ACTIVE) | django.db.models.Q(last_checked=cls.CHECK_REACTIVATE))
 
     @classmethod
     def mbd_get_prepared(cls):
         return cls.objects.filter(status__gte=cls.STATUS_PREPARED)
 
     def mbd_get_check_display(self, typ="string"):
-        if self.mbd_is_checked():
-            return {"string": self.last_checked.strftime("Checked on %Y-%m-%d %H:%M"),
-                    "char": "C"}[typ]
-        else:
-            return self.CHECK_STRINGS[self.last_checked][typ]
+        return {"string": self.last_checked.strftime("Checked on %Y-%m-%d %H:%M"), "char": "C"}[typ] if self.mbd_is_checked() else self.CHECK_STRINGS[self.last_checked][typ]
 
     def mbd_get_status_display(self, typ="string"):
         return "{s} ({p})".format(s=self.get_status_display(),
