@@ -1,7 +1,3 @@
-# -*- coding: utf-8 -*-
-from __future__ import unicode_literals
-from __future__ import absolute_import
-
 import copy
 import os
 import contextlib
@@ -12,7 +8,7 @@ import django.db.models
 import django.contrib.admin
 import django.contrib.messages
 
-import mini_buildd.setup
+import mini_buildd.config
 import mini_buildd.misc
 import mini_buildd.call
 
@@ -68,16 +64,16 @@ go to the default mapping.
              {"classes": ("collapse",),
               "description": """
 <b>Supported extra options</b>
-<p><tt>Debootstrap-Command: ALT_COMMAND</tt>: Alternate command to run instead of standard debootstrap.</p>
+<p><kbd>Debootstrap-Command: ALT_COMMAND</kbd>: Alternate command to run instead of standard debootstrap.</p>
 <p>
-For example, <tt>Debootstrap-Command: /usr/sbin/qemu-debootstrap</tt> may be used to produce <em>armel</em>
-chroots (with <tt>qemu-user-static</tt> installed).
+For example, <kbd>Debootstrap-Command: /usr/sbin/qemu-debootstrap</kbd> may be used to produce <em>armel</em>
+chroots (with <kbd>qemu-user-static</kbd> installed).
 </p>
 """,
               "fields": ("extra_options",)})]
 
         def get_readonly_fields(self, _request, obj=None):
-            "Forbid change source/arch on existing chroot (we would loose the path to the associated data)."
+            """Forbid change source/arch on existing chroot (we would loose the path to the associated data)."""
             fields = copy.copy(self.readonly_fields)
             if obj:
                 fields.append("source")
@@ -88,15 +84,9 @@ chroots (with <tt>qemu-user-static</tt> installed).
         def mbd_host_architecture(cls):
             return mini_buildd.models.source.Architecture.mbd_host_architecture()
 
-        @classmethod
-        def _mbd_get_supported_archs(cls, arch):
-            "Some archs also natively support other archs."
-            arch_map = {"amd64": ["i386"]}
-            return [arch] + arch_map.get(arch, [])
-
         @classmethod
         def _mbd_meta_add_base_sources(cls, chroot_model, msglog):
-            "Add chroot objects for all base sources found."
+            """Add chroot objects for all base sources found."""
             archs = mini_buildd.models.source.Architecture.mbd_supported_architectures()
             msglog.info("Host supports {archs}".format(archs=" ".join(archs)))
 
@@ -105,19 +95,19 @@ chroots (with <tt>qemu-user-static</tt> installed).
                     try:
                         extra_options = ""
                         if s.codename in ["lenny", "etch"]:
-                            extra_options = "Debootstrap-Command: /usr/sbin/mbd-debootstrap-uname-2.6\n"
+                            extra_options = "Debootstrap-Command: /usr/share/mini-buildd/bin/mbd-debootstrap-uname-2.6\n"
                         chroot_model.mbd_get_or_create(msglog, source=s, architecture=a, extra_options=extra_options)
-                    except Exception as e:
-                        msglog.warn("{s}/{a}: Skipping customized chroot (on another backend, or not on default values) [{e}]".format(s=s.codename, a=a.name, e=e))
+                    except BaseException as e:
+                        msglog.warning("{s}/{a}: Skipping customized chroot (on another backend, or not on default values) [{e}]".format(s=s.codename, a=a.name, e=e))
 
-    def __unicode__(self):
+    def __str__(self):
         return "{o} '{c}:{a}' ({f})".format(o=self.source.origin,
                                             c=self.source.codename,
                                             a=self.architecture.name,
                                             f=self.mbd_get_backend().mbd_backend_flavor())
 
     def mbd_get_backend(self):
-        for cls, sub in {"filechroot": [], "dirchroot": [], "lvmchroot": ["looplvmchroot"], "btrfssnapshotchroot": []}.items():
+        for cls, sub in list({"filechroot": [], "dirchroot": [], "lvmchroot": ["looplvmchroot"], "btrfssnapshotchroot": []}.items()):
             if hasattr(self, cls):
                 c = getattr(self, cls)
                 for s in sub:
@@ -127,7 +117,7 @@ chroots (with <tt>qemu-user-static</tt> installed).
         raise Exception("No chroot backend found")
 
     def mbd_get_path(self):
-        return os.path.join(mini_buildd.setup.CHROOTS_DIR, self.source.codename, self.architecture.name)
+        return os.path.join(mini_buildd.config.CHROOTS_DIR, self.source.codename, self.architecture.name)
 
     def mbd_get_name(self):
         return "mini-buildd-{d}-{a}".format(d=self.source.codename, a=self.architecture.name)
@@ -139,14 +129,14 @@ chroots (with <tt>qemu-user-static</tt> installed).
         return os.path.join(self.mbd_get_path(), "schroot.conf")
 
     def mbd_get_keyring_file(self):
-        "Holds all keys from the source to verify the release via debootstrap's --keyring option."
+        """Get keyring file path. Holds all keys from the source to verify the release via debootstrap's --keyring option."""
         return os.path.join(self.mbd_get_path(), "keyring.gpg")
 
     def mbd_get_system_schroot_conf_file(self):
         return os.path.join("/etc/schroot/chroot.d", self.mbd_get_name() + ".conf")
 
     def mbd_get_pre_sequence(self):
-        "Subclasses may implement this to do define an extra preliminary sequence."
+        """Get preliminary sequence. Subclasses may implement this to do define an extra preliminary sequence."""
         LOG.debug("{c}: No pre-sequence defined.".format(c=self))
         return []
 
@@ -155,8 +145,8 @@ chroots (with <tt>qemu-user-static</tt> installed).
         # We should be able to assemble the sequence anyway to be able to remove that chroot.
         try:
             debootstrap_url = self.source.mbd_get_archive().url
-        except:
-            LOG.warning("{c}: Can't get archive URL from source (source removed?).".format(c=self))
+        except BaseException as e:
+            LOG.warning("{c}: Can't get archive URL from source (source removed?): {e}".format(c=self, e=e))
             debootstrap_url = "no_archive_url_found_maybe_source_is_removed"
 
         return [
@@ -174,17 +164,10 @@ chroots (with <tt>qemu-user-static</tt> installed).
                        ["/bin/rm", "--verbose", self.mbd_get_system_schroot_conf_file()])]
 
     def mbd_prepare(self, request):
-        mini_buildd.misc.mkdirs(self.mbd_get_path())
+        os.makedirs(self.mbd_get_path(), exist_ok=True)
 
         # Set personality
-        self.personality = ""
-        if self.personality_override:
-            self.personality = self.personality_override
-        else:
-            try:
-                self.personality = self.PERSONALITIES[self.architecture.name]
-            except:
-                self.personality = "linux"
+        self.personality = self.personality_override if self.personality_override else self.PERSONALITIES.get(self.architecture.name, "linux")
 
         mini_buildd.misc.ConfFile(
             self.mbd_get_schroot_conf_file(),
@@ -225,11 +208,12 @@ personality={p}
     def _mbd_schroot_run(self, call, namespace="chroot", user="root"):
         return mini_buildd.call.Call(["/usr/bin/schroot",
                                       "--chroot", "{n}:{c}".format(n=namespace, c=self.mbd_get_name()),
-                                      "--user", user] +
-                                     call).log().check().ustdout
+                                      "--user", user] + call).log().check().stdout
 
     def mbd_check_sudo_workaround(self, request):
         """
+        Run odd sudo workaround.
+
         mini-buildd <= 1.0.4 created chroots with a "sudo workaround" for bug
         https://bugs.debian.org/cgi-bin/bugreport.cgi?bug=607228.
 
@@ -239,14 +223,14 @@ personality={p}
         try:
             self._mbd_schroot_run(["--directory", "/", "--", "grep", "^{u}".format(u=os.getenv("USER")), "/etc/sudoers"])
             has_sudo_workaround = True
-        except:
+        except BaseException:
             MsgLog(LOG, request).info("{c}: Ok, no sudo workaround found.".format(c=self))
 
         if has_sudo_workaround:
             raise Exception("Chroot has sudo workaround (created with versions <= 1.0.4): Please run 'Remove' + 'PCA' on this chroot to re-create!")
 
     def mbd_backend_check(self, request):
-        "Subclasses may implement this to do extra backend-specific checks."
+        """Run backend check. Subclasses may implement this to do extra backend-specific checks."""
         MsgLog(LOG, request).info("{c}: No backend check implemented.".format(c=self))
 
     def mbd_check(self, request):
@@ -283,8 +267,8 @@ personality={p}
                 MsgLog(LOG, request).log_text(
                     self._mbd_schroot_run(["--directory", "/", "--", "/usr/bin/apt-get", "-q", "-o", "APT::Install-Recommends=false", "--yes"] + args,
                                           namespace="source"))
-            except:
-                MsgLog(LOG, request).warn("'apt-get {args}' not supported in this chroot.".format(args=" ".join(args)))
+            except BaseException as e:
+                MsgLog(LOG, request).warning("'apt-get {args}' not supported in this chroot: {e}".format(args=" ".join(args), e=e))
                 if fatal:
                     raise
 
@@ -293,7 +277,7 @@ personality={p}
 
 
 class DirChroot(Chroot):
-    """ Dir chroot backend. """
+    """Directory chroot backend."""
 
     UNION_AUFS = 0
     UNION_OVERLAYFS = 1
@@ -343,7 +327,7 @@ union-type={u}
 
 
 class FileChroot(Chroot):
-    """ File chroot backend. """
+    """File chroot backend."""
 
     COMPRESSION_NONE = 0
     COMPRESSION_GZIP = 1
@@ -395,16 +379,15 @@ file={t}
             (["/bin/tar",
               "--create",
               "--directory", self.mbd_get_tmp_dir(),
-              "--file", self.mbd_get_tar_file()] +
-             self.TAR_ARGS[self.compression] +
-             ["."],
+              "--file", self.mbd_get_tar_file()] + self.TAR_ARGS[self.compression] + ["."],
              []),
             (["/bin/rm", "--recursive", "--one-file-system", "--force", self.mbd_get_tmp_dir()],
              [])]
 
 
 class LVMChroot(Chroot):
-    """ LVM chroot backend. """
+    """LVM chroot backend."""
+
     volume_group = django.db.models.CharField(max_length=80, default="auto",
                                               help_text="Give a pre-existing LVM volume group name. Just leave it on 'auto' for loop lvm chroots.")
     filesystem = django.db.models.CharField(max_length=10, default="ext2")
@@ -427,7 +410,7 @@ class LVMChroot(Chroot):
     def mbd_get_volume_group(self):
         try:
             return self.looplvmchroot.mbd_get_volume_group()
-        except:
+        except BaseException:
             return self.volume_group
 
     def mbd_get_lvm_device(self):
@@ -462,7 +445,8 @@ lvm-snapshot-options=--size {s}G
 
 
 class LoopLVMChroot(LVMChroot):
-    """ Loop LVM chroot backend. """
+    """Loop LVM chroot backend."""
+
     loop_size = django.db.models.IntegerField(default=100,
                                               help_text="Loop device file size in GB.")
 
@@ -477,8 +461,8 @@ class LoopLVMChroot(LVMChroot):
             cls._mbd_meta_add_base_sources(LoopLVMChroot, msglog)
 
     def mbd_backend_flavor(self):
-        return "{size}G loop: {l}".format(size=self.loop_size,
-                                          l=super(LoopLVMChroot, self).mbd_backend_flavor())
+        return "{size}G loop: {flavor}".format(size=self.loop_size,
+                                               flavor=super().mbd_backend_flavor())
 
     def mbd_get_volume_group(self):
         return "mini-buildd-loop-{d}-{a}".format(d=self.source.codename, a=self.architecture.name)
@@ -488,10 +472,11 @@ class LoopLVMChroot(LVMChroot):
 
     def mbd_get_loop_device(self):
         for f in glob.glob("/sys/block/loop[0-9]*/loop/backing_file"):
-            if os.path.realpath(mini_buildd.misc.open_utf8(f).read().strip()) == os.path.realpath(self.mbd_get_backing_file()):
-                return "/dev/" + f.split("/")[3]
+            with mini_buildd.misc.open_utf8(f) as ld:
+                if os.path.realpath(ld.read().strip()) == os.path.realpath(self.mbd_get_backing_file()):
+                    return "/dev/" + f.split("/")[3]
         LOG.debug("No existing loop device for {b}, searching for free device".format(b=self.mbd_get_backing_file()))
-        return mini_buildd.call.Call(["/sbin/losetup", "--find"], run_as_root=True).log().check().ustdout.rstrip()
+        return mini_buildd.call.Call(["/sbin/losetup", "--find"], run_as_root=True).log().check().stdout.rstrip()
 
     def mbd_get_pre_sequence(self):
         loop_device = self.mbd_get_loop_device()
@@ -510,11 +495,12 @@ class LoopLVMChroot(LVMChroot):
              ["/sbin/pvremove", "--verbose", loop_device]),
 
             (["/sbin/vgcreate", "--verbose", self.mbd_get_volume_group(), loop_device],
-             ["/sbin/vgremove", "--verbose", "--force", self.mbd_get_volume_group()])] + super(LoopLVMChroot, self).mbd_get_pre_sequence()
+             ["/sbin/vgremove", "--verbose", "--force", self.mbd_get_volume_group()])] + super().mbd_get_pre_sequence()
 
 
 class BtrfsSnapshotChroot(Chroot):
-    """ Btrfs Snapshot chroot backend. """
+    """Btrfs Snapshot chroot backend."""
+
     class Meta(Chroot.Meta):
         pass
 
