@@ -1,7 +1,3 @@
-# -*- coding: utf-8 -*-
-from __future__ import unicode_literals
-from __future__ import absolute_import
-
 import re
 import socket
 import email.utils
@@ -13,6 +9,7 @@ import django.core.mail
 import django.contrib.auth.models
 
 import mini_buildd.misc
+import mini_buildd.net
 import mini_buildd.changes
 import mini_buildd.gnupg
 import mini_buildd.builder
@@ -44,7 +41,7 @@ choice.
     hostname = django.db.models.CharField(
         max_length=200,
         default=socket.getfqdn(),
-        help_text="Fully qualified hostname we can be accesed through by others over the network (i.e, from users, uploaders or remotes).")
+        help_text="Fully qualified hostname we can be accessed through by others over the network (i.e, from users, uploaders or remotes).")
 
     email_address = django.db.models.EmailField(max_length=255, default="mini-buildd@{h}".format(h=socket.getfqdn()),
                                                 help_text="""\
@@ -70,11 +67,11 @@ automatically added.""")
     gnupg_keyserver = django.db.models.CharField(
         max_length=200,
         default="pool.sks-keyservers.net",
-        help_text="GnuPG keyserver to use (as fill-in helper).")
+        help_text="GnuPG key server to use (as fill-in helper).")
 
     ftpd_bind = django.db.models.CharField(
         max_length=200,
-        default="0.0.0.0:8067",
+        default=":::8067",
         help_text="FTP Server IP/Hostname and port to bind to.")
 
     ftpd_options = django.db.models.CharField(max_length=255, default="", blank=True, help_text="For future use.")
@@ -86,10 +83,10 @@ automatically added.""")
 
     sbuild_jobs = django.db.models.IntegerField(
         default=1,
-        help_text="Degree of parallelism per build (via DEB_BUILD_OPTIONS).")
+        help_text="Degree of parallelism per build (via sbuild's '--jobs' option).")
 
     # EMail options
-    # DEPRECTATED/UNUSED: With the switch to django mail framework, this is now configured via the --smtp command line argument.
+    # DEPRECATED/UNUSED: With the switch to django mail framework, this is now configured via the --smtp command line argument.
     smtp_server = django.db.models.CharField(
         max_length=254,
         default="localhost:25",
@@ -108,17 +105,17 @@ when this feature is enabled for the resp. repository).
 
 Use '.*' to allow all -- it's however recommended to put this to
 s.th. like '.*@myemail.domain', to avoid original package
-maintainers to be accidentially spammed.
+maintainers to be accidentally spammed.
 """)
 
     custom_hooks_directory = django.db.models.CharField(max_length=255, default="", blank=True, help_text="For future use.")
 
     show_last_packages = django.db.models.IntegerField(
         default=100,
-        help_text="How many 'last packages' data to keep for the status display.")
+        help_text="How many 'last packages data' to keep for the status display.")
     show_last_builds = django.db.models.IntegerField(
         default=100,
-        help_text="How many 'last builds' data to keep for the status display.")
+        help_text="How many 'last builds data' to keep for the status display.")
 
     wait_for_build_results = django.db.models.IntegerField(
         default=5,
@@ -151,11 +148,11 @@ prepare/remove actions will generate/remove the GnuPG key.
         readonly_fields = ["smtp_server", "ftpd_options", "custom_hooks_directory"]
 
         def save_model(self, request, obj, form, change):
-            "Always update date the daemon object to model."
-            obj.mbd_get_daemon().update_to_model(obj)
-            super(Daemon.Admin, self).save_model(request, obj, form, change)
+            """Save model -- always update from daemon object first."""
+            obj.mbd_get_daemon().update_to_model()
+            super().save_model(request, obj, form, change)
 
-    def __unicode__(self):
+    def __str__(self):
         return "{i}: Serving {r} repositories, {c} chroots, using {R} remotes".format(
             i=self.identity,
             r=len(mini_buildd.models.repository.Repository.mbd_get_active()),
@@ -163,11 +160,14 @@ prepare/remove actions will generate/remove the GnuPG key.
             R=len(mini_buildd.models.gnupg.Remote.mbd_get_active()))
 
     def __init__(self, *args, **kwargs):
-        super(Daemon, self).__init__(*args, **kwargs)
+        super().__init__(*args, **kwargs)
         self._mbd_fullname = "mini-buildd archive {i}".format(i=self.identity)
         self._mbd_gnupg = mini_buildd.gnupg.GnuPG(self.gnupg_template, self._mbd_fullname, self.email_address)
         self._mbd_gnupg_long_id = self._mbd_gnupg.get_first_sec_key()
         self._mbd_gnupg_fingerprint = self._mbd_gnupg.get_first_sec_key_fingerprint()
+        interface, port = mini_buildd.net.parse_hopo(self.ftpd_bind)
+        self._mbd_ftp_server_endpoint = mini_buildd.net.ServerEndpoint("tcp6:interface={interface}:port={port}".format(interface=mini_buildd.net.escape(interface), port=port),
+                                                                       mini_buildd.net.Protocol.FTP)
 
     @property
     def mbd_fullname(self):
@@ -185,8 +185,9 @@ prepare/remove actions will generate/remove the GnuPG key.
     def mbd_gnupg_long_id(self):
         return self._mbd_gnupg_long_id
 
-    def clean(self, *args, **kwargs):
-        super(Daemon, self).clean(*args, **kwargs)
+    # Note: pylint false-positive: https://github.com/PyCQA/pylint/issues/1553
+    def clean(self, *args, **kwargs):  # pylint: disable=arguments-differ
+        super().clean(*args, **kwargs)
 
         self.mbd_validate_regex(r"^[a-zA-Z0-9\-]+$", self.identity, "Identity")
 
@@ -201,14 +202,14 @@ prepare/remove actions will generate/remove the GnuPG key.
 
     @classmethod
     def mbd_sync(cls, request):
-        MsgLog(LOG, request).warn("The GnuPG key will never be updated automatically. Explicitly run remove+prepare to achieve this.")
+        MsgLog(LOG, request).warning("The GnuPG key will never be updated automatically. Explicitly run remove+prepare to achieve this.")
 
     def mbd_remove(self, request):
         self._mbd_gnupg.remove()
         MsgLog(LOG, request).info("Daemon GnuPG key removed.")
 
     def mbd_get_dependencies(self):
-        "All active or to-be active repositories, remotes and chroots."
+        """All active or to-be active repositories, remotes and chroots."""
         result = []
         for model in [mini_buildd.models.repository.Repository, mini_buildd.models.chroot.Chroot, mini_buildd.models.gnupg.Remote]:
             for o in model.mbd_get_active_or_auto_reactivate():
@@ -216,21 +217,22 @@ prepare/remove actions will generate/remove the GnuPG key.
         return result
 
     def mbd_check(self, request):
-        "Just warn in case there are no repos and no chroots."
+        """Just warn in case there are no repos and no chroots."""
         if not self.mbd_get_daemon().get_active_repositories() and not self.mbd_get_daemon().get_active_chroots():
-            MsgLog(LOG, request).warn("No active chroot or repository.")
+            MsgLog(LOG, request).warning("No active chroot or repository.")
 
-    def mbd_get_ftp_hopo(self):
-        return mini_buildd.misc.HoPo("{h}:{p}".format(h=self.hostname, p=mini_buildd.misc.HoPo(self.ftpd_bind).port))
+    def mbd_get_ftp_endpoint(self):
+        return self._mbd_ftp_server_endpoint
 
     def mbd_get_ftp_url(self):
-        return "ftp://{h}".format(h=self.mbd_get_ftp_hopo().string)
+        return self._mbd_ftp_server_endpoint.url(self.hostname)
 
-    def mbd_get_http_hopo(self):
-        return mini_buildd.misc.HoPo("{h}:{p}".format(h=self.hostname, p=mini_buildd.misc.HoPo(mini_buildd.setup.HTTPD_BIND).port))
+    @classmethod
+    def mbd_get_http_endpoint(cls):
+        return mini_buildd.config.HTTPD_ENDPOINTS[0]
 
     def mbd_get_http_url(self):
-        return "http://{h}/".format(h=self.mbd_get_http_hopo().string)
+        return mini_buildd.config.HTTPD_ENDPOINTS[0].url(self.hostname)
 
     def mbd_get_archive_origin(self):
         return "Mini-Buildd archive {i} on {h}".format(i=self.identity, h=self.hostname)
@@ -248,7 +250,7 @@ incoming = /incoming
 # These currently only help for mini-buildd-tool bash autocompletion
 x_mini_buildd_host = {H}
 x_mini_buildd_users =
-""".format(i=self.identity, h=self.mbd_get_ftp_hopo().string, H=self.mbd_get_http_hopo().string)
+""".format(i=self.identity, h=self.mbd_get_ftp_endpoint().hopo(self.hostname), H=self.mbd_get_http_endpoint().hopo(self.hostname))
 
     def _mbd_notify_signature(self, typ):
         reason = {"daemon": "Your address is configured to get any notifications (contact administrators if you don't want this).",
@@ -256,7 +258,7 @@ x_mini_buildd_users =
                   "changed-by": "Your address is the uploader of the package ('Changed-By' in changes).",
                   "maintainer": "Your address is the maintainer of the package ('Maintainer' in changes).",
                   "subscriber": "Your user account has a matching subscription.",
-                 }.get(typ, "Unknown")
+                  }.get(typ, "Unknown")
 
         return """
 --\N{SPACE}
@@ -286,25 +288,25 @@ Manage your account : {url}accounts/login/
                     m_to_raw.append(address_raw)
                     msglog.info("Notify: Adding {t} address: {a}".format(t=typ, a=address))
             else:
-                msglog.warn("Notify: Skipping {t} address: {a}: Not allowed (only '{r}')".format(t=typ, a=address, r=self.allow_emails_to))
+                msglog.warning("Notify: Skipping {t} address: {a}: Not allowed (only '{r}')".format(t=typ, a=address, r=self.allow_emails_to))
 
         def get_subscriptions():
             package = changes.get("Source", None)
             real_distribution = distribution
             if real_distribution is None:
-                # If distribution was not given explicitely, try from changes, resolving meta dists if needed.
+                # If distribution was not given explicitly, try from changes, resolving meta dists if needed.
                 changes_dist = changes.get("Distribution", "")
                 real_distribution = mini_buildd.models.repository.map_incoming_distribution(changes_dist)
             return self.mbd_get_daemon().get_subscription_objects().filter(package__in=[package, ""], distribution__in=[real_distribution, ""])
 
         # Add hardcoded addresses from daemon
         for m in self.notify.all():
-            add_to(m.__unicode__(), "daemon", is_automatic=False)
+            add_to(m.__str__(), "daemon", is_automatic=False)
 
         # Add hardcoded addresses from repository
         if repository:
             for m in repository.notify.all():
-                add_to(m.__unicode__(), "repository", is_automatic=False)
+                add_to(m.__str__(), "repository", is_automatic=False)
 
         if changes:
             # Add package uploader (Changed-By): Add even if we do not have a repo, so uploader is informed on such error cases too.
@@ -328,5 +330,5 @@ Manage your account : {url}accounts/login/
         try:
             django.core.mail.send_mass_mail(m_to)
             msglog.info("Notify: Sent '{s}'".format(s=subject))
-        except Exception as e:
-            mini_buildd.setup.log_exception(msglog, "Notify: Mail '{s}' failed to '{r}'".format(s=subject, r=m_to), e)
+        except BaseException as e:
+            mini_buildd.config.log_exception(msglog, "Notify: Mail '{s}' failed to '{r}'".format(s=subject, r=m_to), e)
