@@ -325,7 +325,7 @@ packages (to unstable,experimental,..) aimed for Debian.
 
     def mbd_get_reverse_dependencies(self):
         """When the layout changes, all repos that use that layout also change."""
-        return [r for r in self.repository_set.all()]
+        return list(self.repository_set.all())
 
 
 class ArchitectureOption(mini_buildd.models.base.Model):
@@ -658,7 +658,7 @@ some reason create their automated debug packages with file appendix
 
     def mbd_get_reverse_dependencies(self):
         """When the distribution changes, all repos that use that distribution also change."""
-        return [r for r in self.repository_set.all()]
+        return list(self.repository_set.all())
 
 
 class Repository(mini_buildd.models.base.StatusModel):
