@@ -191,7 +191,7 @@ Use the 'directory' notation with exactly one trailing slash (like 'http://examp
 
     def mbd_get_reverse_dependencies(self):
         """Return all sources (and their deps) that use us."""
-        result = [s for s in self.source_set.all()]
+        result = list(self.source_set.all())
         for s in self.source_set.all():
             result += s.mbd_get_reverse_dependencies()
         return result
@@ -638,7 +638,7 @@ codeversion is only used for base sources.""")
 
     def mbd_get_reverse_dependencies(self):
         """Return all chroots and repositories that use us."""
-        result = [c for c in self.chroot_set.all()]
+        result = list(self.chroot_set.all())
         for d in self.distribution_set.all():
             result += d.mbd_get_reverse_dependencies()
         return result
