@@ -1,7 +1,3 @@
-# -*- coding: utf-8 -*-
-from __future__ import unicode_literals
-from __future__ import absolute_import
-
 import django.db.models
 import django.contrib.auth.models
 
@@ -14,7 +10,7 @@ class Subscription(mini_buildd.models.base.Model):
     package = django.db.models.CharField(max_length=100, blank=True)
     distribution = django.db.models.CharField(max_length=100, blank=True)
 
-    def __unicode__(self):
+    def __str__(self):
         return "User '{u}' subscribes to '{p}' in '{d}'".format(u=self.subscriber,
                                                                 p=self.package if self.package else "any package",
                                                                 d=self.distribution if self.distribution else "any distribution")
