@@ -1,14 +1,11 @@
-# -*- coding: utf-8 -*-
-from __future__ import unicode_literals
-from __future__ import absolute_import
-
 import os
 import shutil
 import logging
 
+import django.utils.timezone
+
 import mini_buildd.misc
 
-import django.utils.timezone
 
 LOG = logging.getLogger(__name__)
 
@@ -22,7 +19,7 @@ class Package(mini_buildd.misc.Status):
     INSTALLED = 10
 
     def __init__(self, daemon, changes):
-        super(Package, self).__init__(
+        super().__init__(
             stati={self.FAILED: "FAILED",
                    self.REJECTED: "REJECTED",
                    self.CHECKING: "CHECKING",
@@ -40,10 +37,10 @@ class Package(mini_buildd.misc.Status):
         self.requests, self.success, self.failed = {}, {}, {}
         self.port_report = {}
 
-    def __unicode__(self):
+    def __str__(self):
         def arch_status():
             result = []
-            for key, _r in self.requests.items():
+            for key, _r in list(self.requests.items()):
                 p = ""
                 if key in self.success:
                     p = "+"
@@ -61,7 +58,7 @@ class Package(mini_buildd.misc.Status):
 
     @property
     def took(self):
-        return round(mini_buildd.misc.timedelta_total_seconds(self.finished - self.started), 1) if self.finished else "n/a"
+        return round((self.finished - self.started).total_seconds(), 1) if self.finished else "n/a"
 
     def precheck(self):
         # Get/check repository, distribution and suite for changes
@@ -81,7 +78,7 @@ class Package(mini_buildd.misc.Status):
 
         # Authenticate
         if self.repository.allow_unauthenticated_uploads:
-            LOG.warn("Unauthenticated uploads allowed. Using '{c}' unchecked".format(c=self.changes.file_name))
+            LOG.warning("Unauthenticated uploads allowed. Using '{c}' unchecked".format(c=self.changes.file_name))
         else:
             self.daemon.keyrings.get_uploaders()[self.repository.identity].verify(self.changes.file_path)
 
@@ -92,15 +89,15 @@ class Package(mini_buildd.misc.Status):
         self.requests = self.changes.gen_buildrequests(self.daemon.model, self.repository, self.distribution, self.suite)
 
         # Upload buildrequests
-        for _key, breq in self.requests.items():
+        for _key, breq in list(self.requests.items()):
             try:
-                breq.upload_buildrequest(self.daemon.model.mbd_get_http_hopo())
-            except Exception as e:
-                mini_buildd.setup.log_exception(LOG,
-                                                "{i}: Buildrequest upload failed".format(i=breq.get_pkg_id()),
-                                                e)
+                breq.upload_buildrequest(self.daemon.model.mbd_get_http_endpoint())
+            except BaseException as e:
+                mini_buildd.config.log_exception(LOG,
+                                                 "{i}: Buildrequest upload failed".format(i=breq.get_pkg_id()),
+                                                 e)
                 # Upload failure build result to ourselves
-                breq.upload_failed_buildresult(self.daemon.model.mbd_gnupg, self.daemon.model.mbd_get_ftp_hopo(), 100, "upload-failed", e)
+                breq.upload_failed_buildresult(self.daemon.model.mbd_gnupg, self.daemon.model.mbd_get_ftp_endpoint(), 100, "upload-failed", e)
 
     def add_buildresult(self, bres):
         self.daemon.keyrings.get_remotes().verify(bres.file_path)
@@ -113,7 +110,7 @@ class Package(mini_buildd.misc.Status):
         status = bres.get("Sbuild-Status")
         lintian = bres.get("Sbuild-Lintian")
 
-        LOG.info("{p}: Got build result for '{a}': {r}={s}, lintian={l}".format(p=self.pid, a=arch, r=retval, s=status, l=lintian))
+        LOG.info("{p}: Got build result for '{a}': {r}={s}, lintian={lintian}".format(p=self.pid, a=arch, r=retval, s=status, lintian=lintian))
 
         def check_lintian(arch):
             return lintian == "pass" or \
@@ -137,7 +134,6 @@ class Package(mini_buildd.misc.Status):
         This may throw on error, and if so, no changes should be
         done to the repo.
         """
-
         # Install to reprepro repository
         self.repository.mbd_package_install(self.distribution, self.suite, self.changes, self.success)
 
@@ -149,13 +145,13 @@ class Package(mini_buildd.misc.Status):
                                  to_dist_str,
                                  self.changes["Version"])
                 self.port_report[to_dist_str] = "Requested"
-            except Exception as e:
+            except BaseException as e:
                 self.port_report[to_dist_str] = "FAILED: {e}".format(e=e)
-                mini_buildd.setup.log_exception(LOG, "{i}: Automatic package port failed for: {d}".format(i=self.changes.get_pkg_id(), d=to_dist_str), e)
+                mini_buildd.config.log_exception(LOG, "{i}: Automatic package port failed for: {d}".format(i=self.changes.get_pkg_id(), d=to_dist_str), e)
 
     def move_to_pkglog(self):
         # Archive build results and request
-        for _arch, c in self.success.items() + self.failed.items() + self.requests.items():
+        for _arch, c in list(self.success.items()) + list(self.failed.items()) + list(self.requests.items()):
             c.move_to_pkglog(self.get_status() == self.INSTALLED, rejected=self.get_status() == self.REJECTED)
         # Archive incoming changes
         self.changes.move_to_pkglog(self.get_status() == self.INSTALLED, rejected=self.get_status() == self.REJECTED)
@@ -185,21 +181,21 @@ class Package(mini_buildd.misc.Status):
                 s=bres.bres_stat,
                 b=os.path.join(self.daemon.model.mbd_get_http_url(),
                                "log",
-                               unicode(bres.get_pkglog_dir(self.get_status() == self.INSTALLED)),
+                               bres.get_pkglog_dir(self.get_status() == self.INSTALLED),
                                bres.buildlog_name))
 
-        results = header(self.__unicode__(), "=")
+        results = header(self.__str__(), "=")
         results += "\n"
 
         if self.failed:
             results += header("Failed builds")
-            for arch, bres in self.failed.items():
+            for arch, bres in list(self.failed.items()):
                 results += bres_result(arch, bres)
             results += "\n"
 
         if self.success:
             results += header("Successful builds")
-            for arch, bres in self.success.items():
+            for arch, bres in list(self.success.items()):
                 results += bres_result(arch, bres)
             results += "\n"
 
@@ -209,29 +205,28 @@ class Package(mini_buildd.misc.Status):
         if self.port_report:
             results += "\n"
             results += header("Port Report")
-            results += "\n".join(("{d:<25}: {r}".format(d=d, r=r) for d, r in self.port_report.items()))
+            results += "\n".join(("{d:<25}: {r}".format(d=d, r=r) for d, r in list(self.port_report.items())))
 
         self.daemon.model.mbd_notify(
-            self.__unicode__(),
+            self.__str__(),
             results,
             self.repository,
             self.changes)
 
 
 class LastPackage(mini_buildd.misc.API):
-    """
-    Subset of 'Package' for pickled statistics.
-    """
+    """Subset of 'Package' for pickled statistics."""
+
     __API__ = -99
 
     def __init__(self, package):
-        super(LastPackage, self).__init__()
+        super().__init__()
 
-        self.identity = package.__unicode__()
+        self.identity = package.__str__()
 
         self.started = package.started
         self.took = package.took
-        self.log = os.path.join("/mini_buildd/log", os.path.dirname(unicode(package.changes.get_pkglog_dir(installed=True))))
+        self.log = os.path.join("/mini_buildd/log", os.path.dirname(package.changes.get_pkglog_dir(installed=True)))
 
         self.changes = {}
         for k in ["source", "distribution", "version"]:
@@ -241,13 +236,13 @@ class LastPackage(mini_buildd.misc.API):
         self.status_desc = package.status_desc
 
         self.requests = {}
-        for a, _r in package.requests.items():
+        for a, _r in list(package.requests.items()):
             self.requests[a] = {}
 
         def cp_bres(src, dst):
-            for a, r in src.items():
+            for a, r in list(src.items()):
                 dst[a] = {"bres_stat": r.bres_stat,
-                          "log": os.path.join("/log", unicode(r.get_pkglog_dir(package.get_status() == package.INSTALLED)), r.buildlog_name)}
+                          "log": os.path.join("/log", r.get_pkglog_dir(package.get_status() == package.INSTALLED), r.buildlog_name)}
 
         self.success = {}
         cp_bres(package.success, self.success)
@@ -255,20 +250,18 @@ class LastPackage(mini_buildd.misc.API):
         self.failed = {}
         cp_bres(package.failed, self.failed)
 
-    def __unicode__(self):
+    def __str__(self):
         return self.identity
 
 
 def package_close(daemon, package):
-    """
-    Close package. Just continue on errors, but log them; guarantee to remove it from the packages dict.
-    """
+    """Close package. Just continue on errors, but log them; guarantee to remove it from the packages dict."""
     try:
         package.move_to_pkglog()
         package.notify()
         daemon.last_packages.appendleft(LastPackage(package))
-    except Exception as e:
-        mini_buildd.setup.log_exception(LOG, "Error closing package '{p}'".format(p=package.pid), e, level=logging.CRITICAL)
+    except BaseException as e:
+        mini_buildd.config.log_exception(LOG, "Error closing package '{p}'".format(p=package.pid), e, level=logging.CRITICAL)
     finally:
         del daemon.packages[package.pid]
 
@@ -288,10 +281,10 @@ def run(daemon, changes):
                 package.install()
                 package.set_status(package.INSTALLED)
                 package_close(daemon, package)
-        except Exception as e:
-            package.set_status(package.FAILED, unicode(e))
+        except BaseException as e:
+            package.set_status(package.FAILED, str(e))
             package_close(daemon, package)
-            mini_buildd.setup.log_exception(LOG, "Package '{p}' FAILED".format(p=pid), e)
+            mini_buildd.config.log_exception(LOG, "Package '{p}' FAILED".format(p=pid), e)
 
     else:  # User upload
         if pid in daemon.packages:
@@ -302,7 +295,7 @@ def run(daemon, changes):
         try:
             package.precheck()
             package.set_status(package.BUILDING)
-        except Exception as e:
-            package.set_status(package.REJECTED, unicode(e))
+        except BaseException as e:
+            package.set_status(package.REJECTED, str(e))
             package_close(daemon, package)
-            mini_buildd.setup.log_exception(LOG, "Package '{p}' REJECTED".format(p=pid), e)
+            mini_buildd.config.log_exception(LOG, "Package '{p}' REJECTED".format(p=pid), e)
