@@ -1,9 +1,4 @@
-# -*- coding: utf-8 -*-
-"""
-Run reprepro commands.
-"""
-from __future__ import unicode_literals
-from __future__ import absolute_import
+"""Run reprepro commands."""
 
 import os
 import shutil
@@ -15,15 +10,14 @@ import mini_buildd.call
 
 LOG = logging.getLogger(__name__)
 
-_LOCKS_LOCK = threading.Lock()
 _LOCKS = {}
 
 
-class Reprepro(object):
+class Reprepro():
     """
     Abstraction to reprepro repository commands.
 
-    Locking
+    *Locking*
 
     This implicitly provides a locking mechanism to avoid
     parallel calls to the same repository from mini-buildd
@@ -32,19 +26,39 @@ class Reprepro(object):
 
     For the case that someone else is using reprepro
     manually, we also always run it with '--waitforlock'.
+
+    *Ignoring 'unusedarch' check*
+
+    Known broken use case is linux' 'make deb-pkg' up to version 4.13.
+
+    linux' native 'make deb-pkg' is the recommended and documented way to
+    produce custom kernels on Debian systems.
+
+    Up to linux version 4.13 (see [#l1]_, [#l2]_), this would also produce
+    firmware packages, flagged "arch=all" in the control file, but
+    actually producing "arch=any" firmware *.deb. The changes file
+    produced however would still list "all" in the Architecture field,
+    making the reprepro "unsusedarch" check fail (and thusly, installation
+    on mini-buildd will fail).
+
+    While this is definitely a bug in 'make deb-pkg' (and also not an
+    issue 4.14 onwards or when you use it w/o producing a firmware
+    package), the check is documented as "safe to ignore" in reprepro, so
+    I think we should allow these cases to work.
+
+    .. [l1] https://github.com/torvalds/linux/commit/cc18abbe449aafc013831a8e0440afc336ae1cba
+    .. [l2] https://github.com/torvalds/linux/commit/5620a0d1aacd554ebebcff373e31107bb1ef7769
     """
+
     def __init__(self, basedir):
         self._basedir = basedir
-        self._cmd = ["reprepro", "--verbose", "--waitforlock", "10", "--basedir", "{b}".format(b=basedir)]
-        # Seems dict.setdefault 'should' be atomic, but it may be not the case in all versions >=2.6
-        # See: http://bugs.python.org/issue13521
-        with _LOCKS_LOCK:
-            self._lock = _LOCKS.setdefault(self._basedir, threading.Lock())
-            LOG.debug("Lock for reprepro repository '{r}': {o}".format(r=self._basedir, o=self._lock))
+        self._cmd = ["reprepro", "--verbose", "--waitforlock", "10", "--ignore", "unusedarch", "--basedir", "{b}".format(b=basedir)]
+        self._lock = _LOCKS.setdefault(self._basedir, threading.Lock())
+        LOG.debug("Lock for reprepro repository '{r}': {o}".format(r=self._basedir, o=self._lock))
 
     def _call(self, args, show_command=False):
         return "{command}{output}".format(command="Running {command}\n".format(command=" ".join(self._cmd + args)) if show_command else "",
-                                          output=mini_buildd.call.Call(self._cmd + args).log().check().ustdout)
+                                          output=mini_buildd.call.Call(self._cmd + args).log().check().stdout)
 
     def _call_locked(self, args, show_command=False):
         with self._lock:
@@ -67,11 +81,11 @@ class Reprepro(object):
     def list(self, pattern, distribution, typ=None, list_max=50):
         result = []
         for item in self._call_locked(["--list-format", "${package}|${$type}|${architecture}|${version}|${$source}|${$sourceversion}|${$codename}|${$component};",
-                                       "--list-max", "{m}".format(m=list_max)] +
-                                      (["--type", "{t}".format(t=typ)] if typ else []) +
-                                      ["listmatched",
-                                       distribution,
-                                       pattern]).split(";"):
+                                       "--list-max", "{m}".format(m=list_max)]
+                                      + (["--type", "{t}".format(t=typ)] if typ else [])
+                                      + ["listmatched",
+                                         distribution,
+                                         pattern]).split(";"):
             if item:
                 item_split = item.split("|")
                 result.append({"package": item_split[0],
@@ -82,7 +96,7 @@ class Reprepro(object):
                                "sourceversion": item_split[5],
                                "distribution": item_split[6],
                                "component": item_split[7],
-                              })
+                               })
         return result
 
     def show(self, package):
@@ -96,7 +110,7 @@ class Reprepro(object):
                 result.append({"source": item_split[0].strip(),
                                "sourceversion": item_split[1].strip(),
                                "distribution": item_split[2].strip(),
-                              })
+                               })
         return result
 
     def migrate(self, package, src_distribution, dst_distribution, version=None):
