@@ -1,16 +1,17 @@
-# -*- coding: utf-8 -*-
-from __future__ import unicode_literals
-from __future__ import absolute_import
-
 import os
-from distutils.version import LooseVersion
+import logging
+import string
+import random
 
 import django
 import django.utils.safestring
 import mini_buildd
+import mini_buildd.api
 
 
-register = django.template.Library()
+LOG = logging.getLogger(__name__)
+
+register = django.template.Library()  # pylint: disable=invalid-name  # django needs this lower-case afaiu
 
 
 @register.filter
@@ -23,13 +24,9 @@ def mbd_dirname(path):
     return os.path.dirname(path)
 
 
-# Use assignment_tag form campat 1.7, 1.8. django >= 1.9 allows this to be a simple_tag
-@register.assignment_tag
+@register.simple_tag
 def mbd_jquery_path():
-    if LooseVersion(django.get_version()) >= LooseVersion("1.9.0"):
-        return "admin/js/vendor/jquery/jquery.js"
-    else:
-        return "admin/js/jquery.js"
+    return "admin/js/vendor/jquery/jquery.js"
 
 
 @register.simple_tag
@@ -38,7 +35,7 @@ def mbd_title():
 
 
 @register.filter
-def mbd_daemon_is_running(dummy):
+def mbd_daemon_is_running(_dummy):
     return mini_buildd.daemon.get().is_running()
 
 
@@ -55,76 +52,31 @@ def mbd_model_count(model):
     return ret
 
 
-@register.inclusion_tag("includes/mbd_packager_status.html")
-def mbd_packager_status(packages):
-    return {"packages": packages}
+@register.inclusion_tag("includes/mbd_api.html", takes_context=True)
+def mbd_api(context, cmd, name=None, title=None, output="html", **kwargs):
+    def _kwargs(prefix):
+        return {k[len(prefix):]: v for k, v in kwargs.items() if k.startswith(prefix)}
 
+    api_cls = mini_buildd.api.COMMANDS_DICT.get(cmd, None)
+    auth_err = api_cls.auth_err(context.get("user"))
+    api_cmd = api_cls(_kwargs("value_"), daemon=mini_buildd.daemon.get())
 
-@register.inclusion_tag("includes/mbd_builder_status.html")
-def mbd_builder_status(builds):
-    return {"builds": builds}
-
-
-@register.inclusion_tag("includes/mbd_manage_subscriptions.html")
-def mbd_manage_subscriptions(repositories, package=""):
-    return {"repositories": repositories,
-            "package": package}
-
-
-@register.inclusion_tag("includes/mbd_admin_index_table_header.html")
-def mbd_admin_index_table_header(title):
-    return {"title": title}
-
-
-@register.inclusion_tag("includes/mbd_admin_index_table_row.html")
-def mbd_admin_index_table_row(app,  # pylint: disable=too-many-arguments
-                              model_name,
-                              model_path,
-                              hide_add=False,
-                              wiz0_function="",
-                              wiz0_name="",
-                              wiz0_title="",
-                              wiz1_function="",
-                              wiz1_name="",
-                              wiz1_title="",
-                              wiz2_function="",
-                              wiz2_name="",
-                              wiz2_title=""):
-    return {"app": app,
-            "model_name": model_name,
-            "model_path": model_path,
-            "hide_add": hide_add,
-            "wiz0_function": wiz0_function,
-            "wiz0_name": wiz0_name,
-            "wiz0_title": wiz0_title,
-            "wiz1_function": wiz1_function,
-            "wiz1_name": wiz1_name,
-            "wiz1_title": wiz1_title,
-            "wiz2_function": wiz2_function,
-            "wiz2_name": wiz2_name,
-            "wiz2_title": wiz2_title}
+    return {"api_cmd": api_cmd,
+            "auth_err": auth_err,
+            "name": name,
+            "title": title,
+            "output": output,
+            "tag_id": "mbd-api-call-{}".format("".join(random.choices(string.ascii_lowercase + string.digits, k=16)))}
 
 
 def _mbd_e2n(func, *args, **kwargs):
     try:
         return func(*args, **kwargs)
-    except:
+    except BaseException as e:
+        LOG.warning("Function failed: {f}: {e}".format(f=func, e=e))
         return None
 
 
-def _mbd_distribution_options(repository, value_prefix="", **suiteoption_filter):
-    result = ""
-    for d in repository.mbd_distribution_strings(**suiteoption_filter):
-        result += '<option value="{p}{d}">{d}</option>'.format(p=value_prefix, d=d)
-    return django.utils.safestring.mark_safe(result)
-
-
-@register.simple_tag
-def mbd_distribution_options(repository, value_prefix="", **suiteoption_filter):
-    "Generate 'option' HTML tags for 'select' form input for distribution strings."
-    return _mbd_e2n(_mbd_distribution_options, repository, value_prefix=value_prefix, **suiteoption_filter)
-
-
 @register.simple_tag
 def mbd_distribution_apt_line(distribution, repository, suite_option):
     return _mbd_e2n(distribution.mbd_get_apt_line, repository, suite_option)
@@ -169,10 +121,10 @@ def mbd_build_status(success, failed):
         lintian_colors = {"pass": "green", "fail": "red", "None": "blue"}
 
         # return "[BL]" html-style colorized
-        result = "[<span style=\"color:{b}\">B</span><span style=\"color:{l}\">L</span>]".format(
-            b=build_colors.get(sbuild_status, "black"),
-            l=lintian_colors.get(lintian_status, "black"))
-    except:
-        pass
+        result = "[<span style=\"color:{bc}\">B</span><span style=\"color:{lc}\">L</span>]".format(
+            bc=build_colors.get(sbuild_status, "black"),
+            lc=lintian_colors.get(lintian_status, "black"))
+    except BaseException as e:
+        LOG.warning("Some error generating build status (ignoring): {e}".format(e=e))
 
     return django.utils.safestring.mark_safe(result)
