@@ -1,6 +1,4 @@
-# -*- coding: utf-8 -*-
-from __future__ import unicode_literals
-
+import os
 import pickle
 import logging
 
@@ -21,17 +19,16 @@ LOG = logging.getLogger(__name__)
 
 
 class AccountProfileView(django.views.generic.base.TemplateView):
-    "This just adds repositories to context."
+    """Add repositories to context."""
+
     def get_context_data(self, **kwargs):
-        context = super(AccountProfileView, self).get_context_data(**kwargs)
+        context = super().get_context_data(**kwargs)
         context.update({"repositories": mini_buildd.models.repository.Repository.mbd_get_prepared()})
         return context
 
 
 def _add_messages(response, msgs):
-    """
-    Add all texts in messages (must be one line each) as custom HTTP headers (using base64 with UTF-8 char encoding).
-    """
+    """Add all texts in messages (must be one line each) as custom HTTP headers (using base64 with UTF-8 char encoding)."""
     n = 0
     for msg in msgs:
         response["X-Mini-Buildd-Message-{n}".format(n=n)] = mini_buildd.misc.u2b64(msg)
@@ -39,14 +36,14 @@ def _add_messages(response, msgs):
 
 
 def _add_api_messages(response, api_cmd, msgs=None):
-    "Add all user messages from api_cmd, plus optional extra messages."
+    """Add all user messages from api_cmd, plus optional extra messages."""
     _add_messages(response,
-                  (api_cmd.msglog.plain.splitlines()[::-1] if api_cmd else []) +
-                  (msgs if msgs else []))
+                  (api_cmd.msglog.plain.splitlines()[::-1] if api_cmd else [])
+                  + (msgs if msgs else []))
 
 
 def _referer(request, output):
-    "output=referer[REFERER_URL]"
+    """output=referer[REFERER_URL]."""
     return output[7:] if (output[:7] == "referer" and output[7:]) else request.META.get("HTTP_REFERER", "/")
 
 
@@ -120,10 +117,14 @@ def home(request):
 def log(request, repository, package, version):
     def get_logs(installed):
         pkg_log = mini_buildd.misc.PkgLog(repository, installed, package, version)
-
-        return {"changes": mini_buildd.misc.open_utf8(pkg_log.changes).read() if pkg_log.changes else None,
-                "changes_path": pkg_log.make_relative(pkg_log.changes) if pkg_log.changes else None,
-                "buildlogs": dict((k, pkg_log.make_relative(v)) for k, v in pkg_log.buildlogs.iteritems())}
+        result = {"changes": None,
+                  "changes_path": None,
+                  "buildlogs": dict((k, pkg_log.make_relative(v)) for k, v in pkg_log.buildlogs.items())}
+        if pkg_log.changes:
+            with mini_buildd.misc.open_utf8(pkg_log.changes) as cf:
+                result["changes"] = cf.read()
+            result["changes_path"] = pkg_log.make_relative(pkg_log.changes)
+        return result
 
     return django.shortcuts.render(request,
                                    "mini_buildd/log.html",
@@ -134,7 +135,21 @@ def log(request, repository, package, version):
                                              ("Failed", get_logs(installed=False))]})
 
 
-def api(request):
+LIVE_BUILDLOGS_404 = """\
+This live buildlog is not yet (or no longer) available.
+
+Please just retry later if this build is currently pending.
+"""
+
+
+def live_buildlogs(_request, logfile):
+    buildlog = os.path.join(mini_buildd.config.SPOOL_DIR, logfile)
+    if not os.path.exists(buildlog):
+        return django.http.HttpResponse(LIVE_BUILDLOGS_404, content_type="text/plain")
+    return django.http.FileResponse(open(buildlog, "rb"), content_type="text/plain")
+
+
+def api(request):  # pylint: disable=too-many-return-statements,too-many-branches
     api_cmd = None
     try:
         if request.method != 'GET':
@@ -154,20 +169,12 @@ def api(request):
         api_cls = mini_buildd.api.COMMANDS_DICT[command]
 
         # Authentication
-        def chk_login():
-            return request.user.is_authenticated() and request.user.is_active
-
-        if (api_cls.AUTH == api_cls.LOGIN) and not chk_login():
-            return error401_unauthorized(request, "API: '{c}': Needs user login".format(c=command))
-
-        if (api_cls.AUTH == api_cls.STAFF) and not (chk_login() and request.user.is_staff):
-            return error401_unauthorized(request, "API: '{c}': Needs staff user login".format(c=command))
-
-        if (api_cls.AUTH == api_cls.ADMIN) and not (chk_login() and request.user.is_superuser):
-            return error401_unauthorized(request, "API: '{c}': Needs superuser login".format(c=command))
+        auth_err = api_cls.auth_err(request.user)
+        if auth_err:
+            return error401_unauthorized(request, auth_err)
 
         # Generate command object
-        api_cmd = api_cls(request.GET, request, msglog=MsgLog(LOG, request))
+        api_cmd = api_cls(request.GET, daemon=mini_buildd.daemon.get(), request=request, msglog=MsgLog(LOG, request))
 
         # HTML output by default
         output = request.GET.get("output", "html")
@@ -178,19 +185,18 @@ def api(request):
 
         # Check confirmable calls
         if api_cls.CONFIRM and request.GET.get("confirm", None) != command:
-            if output == "html" or output == "referer":
+            if output in ("html", "referer"):
                 return django.shortcuts.render(request,
                                                "mini_buildd/api_confirm.html",
                                                {"api_cmd": api_cmd,
                                                 "referer": _referer(request, output)})
-            else:
-                return error401_unauthorized(request, "API: '{c}': Needs to be confirmed".format(c=command))
+            return error401_unauthorized(request, "API: '{c}': Needs to be confirmed".format(c=command))
 
         # Show api command name and user calling it.
         api_cmd.msglog.info("API call '{c}' by user '{u}'".format(c=command, u=request.user))
 
         # Run API call (dep-injection via daemon object)
-        api_cmd.run(mini_buildd.daemon.get())
+        api_cmd.run()
 
         # Generate API call output
         response = None
@@ -202,8 +208,8 @@ def api(request):
                                                 "repositories": mini_buildd.models.repository.Repository.mbd_get_prepared()})
 
         elif output == "plain":
-            response = django.http.HttpResponse(api_cmd.__unicode__().encode(mini_buildd.setup.CHAR_ENCODING),
-                                                content_type="text/plain; charset={charset}".format(charset=mini_buildd.setup.CHAR_ENCODING))
+            response = django.http.HttpResponse(api_cmd.__str__().encode(mini_buildd.config.CHAR_ENCODING),
+                                                content_type="text/plain; charset={charset}".format(charset=mini_buildd.config.CHAR_ENCODING))
 
         elif output == "python":
             response = django.http.HttpResponse(pickle.dumps(api_cmd, pickle.HIGHEST_PROTOCOL),
@@ -211,19 +217,19 @@ def api(request):
 
         elif output[:7] == "referer":
             # Add all plain result lines as info messages on redirect
-            for l in api_cmd.__unicode__().splitlines():
-                api_cmd.msglog.info("Result: {l}".format(l=l))
+            for line in api_cmd.__str__().splitlines():
+                api_cmd.msglog.info("Result: {line}".format(line=line))
             response = django.shortcuts.redirect(_referer(request, output))
         else:
-            response = django.http.HttpResponseBadRequest("<h1>Unknow output type '{o}'</h1>".format(o=output))
+            response = django.http.HttpResponseBadRequest("<h1>Unknown output type '{o}'</h1>".format(o=output))
 
         # Add all user messages as as custom HTTP headers
         _add_api_messages(response, api_cmd)
 
         return response
 
-    except Exception as e:
+    except BaseException as e:
         # This might as well be just an internal error; in case of no bug in the code, 405 fits better though.
         # ['wontfix' unless we refactor to diversified exception classes]
-        mini_buildd.setup.log_exception(LOG, "API call error", e)
+        mini_buildd.config.log_exception(LOG, "API call error", e)
         return error405_method_not_allowed(request, "API call error: {e}".format(e=e), api_cmd=api_cmd)
